@@ -93,6 +93,7 @@ panel_utils_builder_new (XfcePanelPlugin  *panel_plugin,
   panel_return_val_if_fail (XFCE_IS_PANEL_PLUGIN (panel_plugin), NULL);
 
   builder = gtk_builder_new ();
+  gtk_builder_set_translation_domain (builder, GETTEXT_PACKAGE);
   if (gtk_builder_add_from_string (builder, buffer, length, &error))
     {
       dialog = gtk_builder_get_object (builder, "dialog");
@@ -217,3 +218,22 @@ panel_utils_destroy_later (GtkWidget *widget)
   g_idle_add_full (G_PRIORITY_HIGH, destroy_later, widget, NULL);
   g_object_ref_sink (G_OBJECT (widget));
 }
+
+
+
+/* we need to do this when GTK refuses to do it itself, for example to bring back a window
+ * that has been moved off-screen, see https://github.com/wmww/gtk-layer-shell/issues/143 */
+void
+panel_utils_wl_surface_commit (GtkWidget *widget)
+{
+#ifdef HAVE_WAYLAND_CLIENT
+  GdkWindow *window = gtk_widget_get_window (widget);
+  if (window != NULL)
+    {
+      /* yes, it can be null when the window is not */
+      struct wl_surface *wl_surface = gdk_wayland_window_get_wl_surface (window);
+      if (wl_surface != NULL)
+        wl_surface_commit (wl_surface);
+    }
+#endif
+}
