@@ -41,7 +41,9 @@
 #include <gio/gio.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
+#ifdef HAVE_LIBWNCK
 #include <libwnck/libwnck.h>
+#endif
 
 #include <common/panel-private.h>
 #include <common/panel-debug.h>
@@ -205,7 +207,7 @@ panel_dbus_name_lost (GDBusConnection *connection,
                       gpointer         user_data)
 {
   if (connection == NULL)
-    g_critical (_("Name %s lost on the message dbus, exiting."), name);
+    g_critical ("Name %s lost on the message dbus, exiting.", name);
 
   gtk_main_quit ();
 }
@@ -251,6 +253,12 @@ main (gint argc, gchar **argv)
   /* set translation domain */
   xfce_textdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR, "UTF-8");
 
+#ifdef G_ENABLE_DEBUG
+  /* do NOT remove this line for now, If something doesn't work,
+   * fix your code instead! */
+  g_log_set_always_fatal (G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_WARNING);
+#endif
+
   /* Workaround for xinput2's subpixel handling triggering unwanted enter/leave-notify events:
    * https://bugs.freedesktop.org/show_bug.cgi?id=92681
    * We retain the original env var in our own custom env var which we use to re-set the
@@ -287,7 +295,7 @@ main (gint argc, gchar **argv)
       else
         g_print ("%s", PACKAGE_NAME);
       g_print (" %s (Xfce %s)\n\n", PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2004-2024");
+      g_print ("%s\n", "Copyright (c) 2004-2023");
       g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
       g_print ("\n");
@@ -369,9 +377,12 @@ main (gint argc, gchar **argv)
     signal (signums[i], panel_signal_handler);
 
   /* set EWMH source indication */
+#ifdef HAVE_LIBWNCK
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-  wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
+  if (GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+    wnck_set_client_type (WNCK_CLIENT_TYPE_PAGER);
 G_GNUC_END_IGNORE_DEPRECATIONS
+#endif
 
   gtk_main ();
 
