@@ -29,7 +29,13 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
 
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_GTK_LAYER_SHELL
+#include <gtk-layer-shell/gtk-layer-shell.h>
+#else
+#define gtk_layer_is_supported() FALSE
+#endif
+
+#ifdef HAVE_LIBX11
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #endif
@@ -52,12 +58,14 @@
 #include <panel/panel-dialogs.h>
 #include <panel/panel-plugin-external.h>
 
-#define MIGRATE_BIN HELPERDIR G_DIR_SEPARATOR_S "migrate"
+#define AUTOSAVE_INTERVAL   (10 * 60)
+#define MIGRATE_BIN         HELPERDIR G_DIR_SEPARATOR_S "migrate"
 
 
 
 static void      panel_application_dispose            (GObject                *object);
 static void      panel_application_finalize           (GObject                *object);
+static gboolean  panel_application_autosave_timer     (gpointer                user_data);
 static void      panel_application_plugin_move        (GtkWidget              *item,
                                                        PanelApplication       *application);
 static gboolean  panel_application_plugin_insert      (PanelApplication       *application,
@@ -122,10 +130,13 @@ struct _PanelApplication
   /* internal list of opened dialogs */
   GSList             *dialogs;
 
+  /* autosave timer for plugins */
+  guint               autosave_timer_id;
+
   /* autohide count at application level */
   gint                autohide_block;
 
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_LIBX11
   guint               wait_for_wm_timeout_id;
 #endif
 
@@ -136,7 +147,7 @@ struct _PanelApplication
   guint               drop_index;
 };
 
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_LIBX11
 typedef struct
 {
   PanelApplication *application;
@@ -193,8 +204,9 @@ panel_application_class_init (PanelApplicationClass *klass)
 static void
 panel_application_init (PanelApplication *application)
 {
+  GdkDisplay *display;
   GError *error = NULL;
-  gint    configver;
+  gint configver;
 
   application->windows = NULL;
   application->dialogs = NULL;
@@ -217,12 +229,34 @@ panel_application_init (PanelApplication *application)
         }
     }
 
-  /* check if we need to force all plugins to run external */
-  if (xfconf_channel_get_bool (application->xfconf, "/force-all-external", FALSE))
-    panel_module_factory_force_all_external ();
+  /* check if we need to force all plugins to run internal/external */
+  if (xfconf_channel_get_bool (application->xfconf, "/force-all-internal", FALSE))
+    panel_module_factory_force_run_mode (PANEL_MODULE_RUN_MODE_INTERNAL);
+  else if (xfconf_channel_get_bool (application->xfconf, "/force-all-external", FALSE))
+    panel_module_factory_force_run_mode (PANEL_MODULE_RUN_MODE_EXTERNAL);
 
   /* get a factory reference so it never unloads */
   application->factory = panel_module_factory_get ();
+
+  /* start the autosave timer for plugins */
+  application->autosave_timer_id = g_timeout_add_seconds (60 * 10,
+      panel_application_autosave_timer, application);
+
+  /* warn the user about restricted features on Wayland */
+  display = gdk_display_get_default ();
+  if (GDK_IS_WAYLAND_DISPLAY (display))
+    {
+      if (! gtk_layer_is_supported ())
+        g_warning ("Wayland detected without layer-shell support (either because of your"
+                   " compositor or because Xfce4-panel was built without this support):"
+                   " Xfce4-panel might not look like a panel and many of its features will"
+                   " not be available");
+      if (! gdk_wayland_display_query_registry (display, "zwlr_foreign_toplevel_manager_v1"))
+        g_warning ("Wayland detected without foreign-toplevel-management support (your"
+                   " compositor does not seem to support it): Some Xfce4-panel features will"
+                   " not work (e.g. intellihide), as well as some plugins (e.g. ShowDesktop,"
+                   " Tasklist, WindowMenu)");
+    }
 }
 
 
@@ -234,7 +268,12 @@ panel_application_dispose (GObject *object)
 
   /* save plugins: xfconf_shutdown() is called via a weak ref i.e. on dispose(),
    * so this should be done here to avoid any use-after-free */
-  panel_application_save (application, SAVE_PLUGIN_PROVIDERS);
+  if (application->autosave_timer_id != 0)
+    {
+      g_source_remove (application->autosave_timer_id);
+      application->autosave_timer_id = 0;
+      panel_application_autosave_timer (application);
+    }
 
   (*G_OBJECT_CLASS (panel_application_parent_class)->dispose) (object);
 }
@@ -248,7 +287,7 @@ panel_application_finalize (GObject *object)
 
   panel_return_if_fail (application->dialogs == NULL);
 
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_LIBX11
   /* stop autostart timeout */
   if (application->wait_for_wm_timeout_id != 0)
     g_source_remove (application->wait_for_wm_timeout_id);
@@ -268,6 +307,19 @@ panel_application_finalize (GObject *object)
 
 
 
+static gboolean
+panel_application_autosave_timer (gpointer user_data)
+{
+  PanelApplication *application = PANEL_APPLICATION (user_data);
+
+  /* emit a save signal for the plugins */
+  panel_application_save (application, SAVE_PLUGIN_PROVIDERS);
+
+  return TRUE;
+}
+
+
+
 static void
 panel_application_xfconf_window_bindings (PanelApplication *application,
                                           PanelWindow      *window,
@@ -293,7 +345,7 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "icon-size", G_TYPE_UINT },
     { "output-name", G_TYPE_STRING },
     { "position", G_TYPE_STRING },
-    { "enable-struts", G_TYPE_BOOLEAN },
+    { "disable-struts", G_TYPE_BOOLEAN },
     { NULL }
   };
   const PanelProperty  global_properties[] =
@@ -301,26 +353,14 @@ panel_application_xfconf_window_bindings (PanelApplication *application,
     { "dark-mode", G_TYPE_BOOLEAN },
     { NULL }
   };
-  const PanelProperty  old_properties[] =
-  {
-    { "autohide", G_TYPE_BOOLEAN },
-    { "disable-struts", G_TYPE_BOOLEAN },
-    { NULL }
-  };
-  const PanelProperty  new_properties[] =
-  {
-    { "autohide-behavior", G_TYPE_UINT },
-    { "enable-struts", G_TYPE_BOOLEAN },
-    { NULL }
-  };
 
   panel_return_if_fail (XFCONF_IS_CHANNEL (application->xfconf));
 
   /* create the property base */
   property_base = g_strdup_printf (PANELS_PROPERTY_BASE, panel_window_get_id (window));
 
-  /* migrate old properties */
-  panel_window_migrate_old_properties (window, application->xfconf, property_base, old_properties, new_properties);
+  /* migrate old autohide property */
+  panel_window_migrate_autohide_property (window, application->xfconf, property_base);
 
   /* bind all the properties */
   panel_properties_bind (application->xfconf, G_OBJECT (window),
@@ -464,7 +504,7 @@ panel_application_load_real (PanelApplication *application)
 
 
 
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_LIBX11
 static gboolean
 panel_application_wait_for_window_manager (gpointer data)
 {
@@ -585,7 +625,7 @@ panel_application_plugin_move (GtkWidget        *item,
   module = panel_module_get_from_plugin_provider (XFCE_PANEL_PLUGIN_PROVIDER (item));
   icon_name = panel_module_get_icon_name (module);
   theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (item));
-  if (!panel_str_is_empty (icon_name)
+  if (!xfce_str_is_empty (icon_name)
       && gtk_icon_theme_has_icon (theme, icon_name))
     gtk_drag_set_icon_name (context, icon_name, 0, 0);
   else
@@ -609,7 +649,7 @@ panel_application_plugin_delete_config (PanelApplication *application,
   gchar *filename, *path;
 
   panel_return_if_fail (PANEL_IS_APPLICATION (application));
-  panel_return_if_fail (!panel_str_is_empty (name));
+  panel_return_if_fail (!xfce_str_is_empty (name));
   panel_return_if_fail (unique_id != -1);
 
   /* remove the xfconf property */
@@ -666,7 +706,22 @@ panel_application_plugin_provider_signal (XfcePanelPluginProvider       *provide
       /* check the window locking, not that of the provider, because
        * the users might have worked around that and both should be identical */
       if (!panel_window_get_locked (window))
-        panel_application_plugin_move (GTK_WIDGET (provider), application);
+        {
+          /* widget dnd doesn't seem to work on Wayland without holding down a mouse button,
+           * which leads to an unsolvable problem in GTK 3 because the plugin can't catch
+           * the event before its child widgets (no "capture" phase as in GTK 4) */
+          if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+            {
+              gint item;
+              itembar = gtk_bin_get_child (GTK_BIN (window));
+              item = panel_itembar_get_child_index (PANEL_ITEMBAR (itembar), GTK_WIDGET (provider));
+              g_object_set_data_full (G_OBJECT (window), "prefs-dialog-item",
+                                      g_strdup_printf ("%d", item), g_free);
+              panel_preferences_dialog_show (window);
+            }
+          else
+            panel_application_plugin_move (GTK_WIDGET (provider), application);
+        }
       break;
 
     case PROVIDER_SIGNAL_EXPAND_PLUGIN:
@@ -1191,13 +1246,13 @@ gboolean
 panel_application_load (PanelApplication  *application,
                         gboolean           disable_wm_check)
 {
-#ifdef GDK_WINDOWING_X11
+#ifdef HAVE_LIBX11
   Display    *display;
   WaitForWM  *wfwm;
   guint       i;
   gchar     **atom_names;
 
-  if (!disable_wm_check)
+  if (!disable_wm_check && GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
     {
       display = XOpenDisplay (NULL);
       if (display == NULL)
