@@ -25,6 +25,11 @@
 #endif
 
 #include <gtk/gtk.h>
+#ifdef HAVE_GTK_LAYER_SHELL
+#include <gtk-layer-shell/gtk-layer-shell.h>
+#else
+#define gtk_layer_is_supported() FALSE
+#endif
 
 #include <libxfce4panel/libxfce4panel.h>
 #include <libxfce4panel/xfce-panel-plugin-provider.h>
@@ -185,15 +190,6 @@ panel_base_window_class_init (PanelBaseWindowClass *klass)
 
 
 
-static void
-panel_base_window_scale_factor_changed (PanelBaseWindow *window)
-{
-  if (window->background_style == PANEL_BG_STYLE_IMAGE && window->background_image != NULL)
-    panel_base_window_set_background_image_css (window);
-}
-
-
-
 static void
 panel_base_window_init (PanelBaseWindow *window)
 {
@@ -229,7 +225,6 @@ panel_base_window_init (PanelBaseWindow *window)
   context = gtk_widget_get_style_context (GTK_WIDGET (window));
   gtk_style_context_add_class (context, "panel");
   gtk_style_context_add_class (context, "xfce4-panel");
-  g_signal_connect (window, "notify::scale-factor", G_CALLBACK (panel_base_window_scale_factor_changed), NULL);
 }
 
 
@@ -308,7 +303,6 @@ panel_base_window_set_property (GObject      *object,
   PanelBgStyle            bg_style;
   GFile                  *file;
   gdouble                 opacity;
-  const gchar            *str;
 
   switch (prop_id)
     {
@@ -385,17 +379,11 @@ panel_base_window_set_property (GObject      *object,
       break;
 
     case PROP_BACKGROUND_IMAGE:
+      /* store new uri, built and escaped through a GFile */
       g_free (window->background_image);
-      str = g_value_get_string (value);
-      if (str != NULL)
-        {
-          /* store new uri, built and escaped through a GFile */
-          file = g_file_new_for_commandline_arg (str);
-          window->background_image = g_file_get_uri (file);
-          g_object_unref (file);
-        }
-      else
-        window->background_image = NULL;
+      file = g_file_new_for_commandline_arg (g_value_get_string (value));
+      window->background_image = g_file_get_uri (file);
+      g_object_unref (file);
 
       if (window->background_style == PANEL_BG_STYLE_IMAGE)
         {
@@ -474,11 +462,9 @@ panel_base_window_screen_changed (GtkWidget *widget, GdkScreen *previous_screen)
   /* set the rgba colormap if supported by the screen */
   screen = gtk_window_get_screen (GTK_WINDOW (window));
   visual = gdk_screen_get_rgba_visual (screen);
-  if (visual != NULL)
-    {
-      gtk_widget_set_visual (widget, visual);
-      window->is_composited = gdk_screen_is_composited (screen);
-    }
+  window->is_composited = gdk_screen_is_composited (screen);
+  if (visual != NULL && window->is_composited)
+    gtk_widget_set_visual (widget, visual);
 
    panel_debug (PANEL_DEBUG_BASE_WINDOW,
                "%p: rgba visual=%p, compositing=%s", window,
@@ -608,29 +594,17 @@ panel_base_window_set_background_color_css (PanelBaseWindow *window)
 static void
 panel_base_window_set_background_image_css (PanelBaseWindow *window)
 {
-  gchar *css_url, *css_string;
-  gint scale_factor;
+  gchar *css_string;
 
   panel_return_if_fail (window->background_image != NULL);
 
-  /* do not scale background image with the panel */
-  scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (window));
-  css_url = g_strdup_printf ("url(\"%s\")", window->background_image);
-  for (gint i = 1; i < scale_factor; i++)
-    {
-      gchar *temp = g_strdup_printf ("%s,url(\"%s\")", css_url, window->background_image);
-      g_free (css_url);
-      css_url = temp;
-    }
-
-  css_string = g_strdup_printf (".xfce4-panel.background { background: -gtk-scaled(%s);"
+  css_string = g_strdup_printf (".xfce4-panel.background { background: url(\"%s\");"
                                                           "border-color: transparent; } %s",
-                                css_url, PANEL_BASE_CSS);
+                                window->background_image, PANEL_BASE_CSS);
 
   panel_base_window_set_background_css (window, css_string);
 
   g_free (css_string);
-  g_free (css_url);
 }
 
 
@@ -811,23 +785,6 @@ panel_base_window_set_plugin_background_image (GtkWidget *widget,
 
 
 
-void
-panel_base_window_move_resize (PanelBaseWindow *window,
-                               gint             x,
-                               gint             y,
-                               gint             width,
-                               gint             height)
-{
-  panel_return_if_fail (PANEL_IS_BASE_WINDOW (window));
-
-  if (width > 0 && height > 0)
-    gtk_window_resize (GTK_WINDOW (window), width, height);
-
-  gtk_window_move (GTK_WINDOW (window), x, y);
-}
-
-
-
 void
 panel_base_window_set_borders (PanelBaseWindow *window,
                                PanelBorders     borders)
@@ -885,6 +842,10 @@ panel_base_window_opacity_enter (PanelBaseWindow *window,
     }
   else
     {
+      /* needs a recheck when timeout is over on Wayland, see panel_window_pointer_is_outside() */
+      if (gtk_layer_is_supported () && ! panel_window_pointer_is_outside (PANEL_WINDOW (window)))
+        return;
+
       gtk_widget_set_opacity (GTK_WIDGET (window), window->leave_opacity);
       panel_base_window_set_plugin_data (window, panel_base_window_set_plugin_leave_opacity);
     }
