@@ -526,7 +526,7 @@ panel_item_dialog_drag_begin (GtkWidget       *treeview,
           /* set the drag icon */
           icon_name = panel_module_get_icon_name (module);
           theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (treeview));
-          if (!panel_str_is_empty (icon_name)
+          if (!xfce_str_is_empty (icon_name)
               && gtk_icon_theme_has_icon (theme, icon_name))
             gtk_drag_set_icon_name (context, icon_name, 0, 0);
           else
@@ -612,9 +612,6 @@ panel_item_dialog_populate_store (PanelItemDialog *dialog)
   panel_return_if_fail (PANEL_IS_MODULE_FACTORY (dialog->factory));
   panel_return_if_fail (GTK_IS_LIST_STORE (dialog->store));
 
-  /* clear previous entries if needed */
-  gtk_list_store_clear (dialog->store);
-
   /* add all known modules in the factory */
   modules = panel_module_factory_get_modules (dialog->factory);
   for (li = modules, n = 0; li != NULL; li = li->next, n++)
@@ -725,7 +722,7 @@ panel_item_dialog_visible_func (GtkTreeModel *model,
 
   /* search string from dialog */
   text = gtk_entry_get_text (entry);
-  if (G_UNLIKELY (panel_str_is_empty (text)))
+  if (G_UNLIKELY (xfce_str_is_empty (text)))
     return TRUE;
 
   gtk_tree_model_get (model, iter, COLUMN_MODULE, &module, -1);
@@ -795,7 +792,7 @@ panel_item_dialog_text_renderer (GtkTreeViewColumn *column,
 
   /* avoid (null) in markup string */
   comment = panel_module_get_comment (module);
-  if (panel_str_is_empty (comment))
+  if (xfce_str_is_empty (comment))
     comment = "";
 
   name = panel_module_get_display_name (module);
