@@ -23,6 +23,11 @@
 #include <gmodule.h>
 #include <glib/gstdio.h>
 #include <libxfce4util/libxfce4util.h>
+#ifdef HAVE_GTK_LAYER_SHELL
+#include <gtk-layer-shell/gtk-layer-shell.h>
+#else
+#define gtk_layer_is_supported() FALSE
+#endif
 
 #include <common/panel-private.h>
 #include <common/panel-debug.h>
@@ -37,12 +42,12 @@
 #define PANEL_PLUGINS_LIB_DIR_OLD (LIBDIR G_DIR_SEPARATOR_S "panel-plugins")
 
 
-typedef enum _PanelModuleRunMode PanelModuleRunMode;
 typedef enum _PanelModuleUnique  PanelModuleUnique;
 
 
 
 static void      panel_module_dispose          (GObject          *object);
+static void      panel_module_finalize         (GObject          *object);
 static gboolean  panel_module_load             (GTypeModule      *type_module);
 static void      panel_module_unload           (GTypeModule      *type_module);
 static void      panel_module_plugin_destroyed (gpointer          user_data,
@@ -55,13 +60,6 @@ struct _PanelModuleClass
   GTypeModuleClass __parent__;
 };
 
-enum _PanelModuleRunMode
-{
-  UNKNOWN,    /* Unset */
-  INTERNAL,   /* plugin library will be loaded in the panel */
-  WRAPPER     /* external library with communication through PanelPluginExternal */
-};
-
 enum _PanelModuleUnique
 {
   UNIQUE_FALSE,
@@ -119,6 +117,7 @@ panel_module_class_init (PanelModuleClass *klass)
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->dispose = panel_module_dispose;
+  gobject_class->finalize = panel_module_finalize;
 
   gtype_module_class = G_TYPE_MODULE_CLASS (klass);
   gtype_module_class->load = panel_module_load;
@@ -132,7 +131,7 @@ panel_module_class_init (PanelModuleClass *klass)
 static void
 panel_module_init (PanelModule *module)
 {
-  module->mode = UNKNOWN;
+  module->mode = PANEL_MODULE_RUN_MODE_NONE;
   module->filename = NULL;
   module->display_name = NULL;
   module->comment = NULL;
@@ -149,26 +148,29 @@ panel_module_init (PanelModule *module)
 
 static void
 panel_module_dispose (GObject *object)
+{
+  /* Do nothing to avoid problems with dispose in GTypeModule when
+   * types are registered.
+   *
+   * For us this is not a problem since the modules are released when
+   * everything is destroyed. So we really want that last unref before
+   * closing the application. */
+}
+
+
+
+static void
+panel_module_finalize (GObject *object)
 {
   PanelModule *module = PANEL_MODULE (object);
 
-  if (module->api != NULL)
-    {
-      g_free (module->filename);
-      g_free (module->display_name);
-      g_free (module->comment);
-      g_free (module->icon_name);
-      g_free (module->api);
-      module->api = NULL;
-      if (module->plugin_type != G_TYPE_NONE)
-        {
-          /* a module containing type implementations must exist forever */
-          g_object_ref (module);
-          return;
-        }
-    }
+  g_free (module->filename);
+  g_free (module->display_name);
+  g_free (module->comment);
+  g_free (module->icon_name);
+  g_free (module->api);
 
-  G_OBJECT_CLASS (panel_module_parent_class)->dispose (object);
+  (*G_OBJECT_CLASS (panel_module_parent_class)->finalize) (object);
 }
 
 
@@ -183,7 +185,7 @@ panel_module_load (GTypeModule *type_module)
 
   panel_return_val_if_fail (PANEL_IS_MODULE (module), FALSE);
   panel_return_val_if_fail (G_IS_TYPE_MODULE (module), FALSE);
-  panel_return_val_if_fail (module->mode == INTERNAL, FALSE);
+  panel_return_val_if_fail (module->mode == PANEL_MODULE_RUN_MODE_INTERNAL, FALSE);
   panel_return_val_if_fail (module->library == NULL, FALSE);
   panel_return_val_if_fail (module->plugin_type == G_TYPE_NONE, FALSE);
   panel_return_val_if_fail (module->construct_func == NULL, FALSE);
@@ -210,7 +212,7 @@ panel_module_load (GTypeModule *type_module)
       panel_module_unload (type_module);
 
       /* from now on, run this plugin in a wrapper */
-      module->mode = WRAPPER;
+      module->mode = PANEL_MODULE_RUN_MODE_EXTERNAL;
       g_free (module->api);
       module->api = g_strdup (LIBXFCE4PANEL_VERSION_API);
 
@@ -250,7 +252,7 @@ panel_module_unload (GTypeModule *type_module)
 
   panel_return_if_fail (PANEL_IS_MODULE (module));
   panel_return_if_fail (G_IS_TYPE_MODULE (module));
-  panel_return_if_fail (module->mode == INTERNAL);
+  panel_return_if_fail (module->mode == PANEL_MODULE_RUN_MODE_INTERNAL);
   panel_return_if_fail (module->library != NULL);
   panel_return_if_fail (module->plugin_type != G_TYPE_NONE
                         || module->construct_func != NULL);
@@ -279,7 +281,7 @@ panel_module_plugin_destroyed (gpointer  user_data,
   module->use_count--;
 
   /* unuse the library if the plugin runs internal */
-  if (module->mode == INTERNAL)
+  if (module->mode == PANEL_MODULE_RUN_MODE_INTERNAL)
     g_type_module_unuse (G_TYPE_MODULE (module));
 
   /* emit signal unique signal in the factory */
@@ -292,7 +294,7 @@ panel_module_plugin_destroyed (gpointer  user_data,
 PanelModule *
 panel_module_new_from_desktop_file (const gchar *filename,
                                     const gchar *name,
-                                    gboolean     force_external)
+                                    PanelModuleRunMode forced_mode)
 {
   PanelModule *module = NULL;
   XfceRc      *rc;
@@ -301,8 +303,8 @@ panel_module_new_from_desktop_file (const gchar *filename,
   const gchar *module_unique;
   gboolean     found;
 
-  panel_return_val_if_fail (!panel_str_is_empty (filename), NULL);
-  panel_return_val_if_fail (!panel_str_is_empty (name), NULL);
+  panel_return_val_if_fail (!xfce_str_is_empty (filename), NULL);
+  panel_return_val_if_fail (!xfce_str_is_empty (name), NULL);
 
   rc = xfce_rc_simple_open (filename, TRUE);
   if (G_UNLIKELY (rc == NULL))
@@ -363,15 +365,18 @@ panel_module_new_from_desktop_file (const gchar *filename,
           module->filename = path;
 
           /* run mode of the module, by default everything runs in
-           * the wrapper, unless defined otherwise */
-          if (force_external || !xfce_rc_read_bool_entry (rc, "X-XFCE-Internal", FALSE))
+           * the wrapper, unless defined otherwise or unsupported */
+          if (forced_mode != PANEL_MODULE_RUN_MODE_INTERNAL
+              && (GDK_IS_X11_DISPLAY (gdk_display_get_default ()) || gtk_layer_is_supported ())
+              && (forced_mode == PANEL_MODULE_RUN_MODE_EXTERNAL
+                  || !xfce_rc_read_bool_entry (rc, "X-XFCE-Internal", FALSE)))
             {
-              module->mode = WRAPPER;
+              module->mode = PANEL_MODULE_RUN_MODE_EXTERNAL;
               g_free (module->api);
               module->api = g_strdup (xfce_rc_read_entry (rc, "X-XFCE-API", LIBXFCE4PANEL_VERSION_API));
             }
           else
-            module->mode = INTERNAL;
+            module->mode = PANEL_MODULE_RUN_MODE_INTERNAL;
         }
       else
         {
@@ -384,7 +389,7 @@ panel_module_new_from_desktop_file (const gchar *filename,
   if (G_LIKELY (module != NULL))
     {
       g_type_module_set_name (G_TYPE_MODULE (module), name);
-      panel_assert (module->mode != UNKNOWN);
+      panel_assert (module->mode != PANEL_MODULE_RUN_MODE_NONE);
 
       /* read the remaining information */
       module->display_name = g_strdup (xfce_rc_read_entry (rc, "Name", name));
@@ -403,7 +408,7 @@ panel_module_new_from_desktop_file (const gchar *filename,
 
        panel_debug_filtered (PANEL_DEBUG_MODULE, "new module %s, filename=%s, internal=%s",
                              name, module->filename,
-                             PANEL_DEBUG_BOOL (module->mode == INTERNAL));
+                             PANEL_DEBUG_BOOL (module->mode == PANEL_MODULE_RUN_MODE_INTERNAL));
     }
 
   xfce_rc_close (rc);
@@ -426,7 +431,7 @@ panel_module_new_plugin (PanelModule  *module,
   panel_return_val_if_fail (G_IS_TYPE_MODULE (module), NULL);
   panel_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
   panel_return_val_if_fail (unique_id != -1, NULL);
-  panel_return_val_if_fail (module->mode != UNKNOWN, NULL);
+  panel_return_val_if_fail (module->mode != PANEL_MODULE_RUN_MODE_NONE, NULL);
 
   /* return null if the module is not usable (unique and already used) */
   if (G_UNLIKELY (!panel_module_is_usable (module, screen)))
@@ -434,7 +439,7 @@ panel_module_new_plugin (PanelModule  *module,
 
   switch (module->mode)
     {
-    case INTERNAL:
+    case PANEL_MODULE_RUN_MODE_INTERNAL:
       if (g_type_module_use (G_TYPE_MODULE (module)))
         {
           if (module->plugin_type != G_TYPE_NONE)
@@ -474,7 +479,7 @@ panel_module_new_plugin (PanelModule  *module,
        * note: next comment tells GCC7 to ignore the fallthrough */
       /* fall through */
 
-    case WRAPPER:
+    case PANEL_MODULE_RUN_MODE_EXTERNAL:
       plugin = panel_plugin_external_wrapper_new (module, unique_id, arguments);
       debug_type = "external-wrapper";
       break;
