@@ -31,7 +31,6 @@
 #endif
 
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <libxfce4util/libxfce4util.h>
 
 #include <gio/gio.h>
@@ -61,12 +60,8 @@ static void         panel_plugin_external_set_property            (GObject
                                                                    guint                             prop_id,
                                                                    const GValue                     *value,
                                                                    GParamSpec                       *pspec);
-static void         panel_plugin_external_size_allocate           (GtkWidget                        *widget,
-                                                                   GtkAllocation                    *allocation);
 static void         panel_plugin_external_realize                 (GtkWidget                        *widget);
 static void         panel_plugin_external_unrealize               (GtkWidget                        *widget);
-static void         panel_plugin_external_plug_added              (GtkSocket                        *socket);
-static gboolean     panel_plugin_external_plug_removed            (GtkSocket                        *socket);
 static gboolean     panel_plugin_external_child_ask_restart       (PanelPluginExternal              *external);
 static void         panel_plugin_external_child_spawn             (PanelPluginExternal              *external);
 static void         panel_plugin_external_child_respawn_schedule  (PanelPluginExternal              *external);
@@ -76,11 +71,6 @@ static void         panel_plugin_external_child_watch             (GPid
 static void         panel_plugin_external_child_watch_destroyed   (gpointer                          user_data);
 static void         panel_plugin_external_queue_free              (PanelPluginExternal              *external);
 static void         panel_plugin_external_queue_send_to_child     (PanelPluginExternal              *external);
-static void         panel_plugin_external_queue_add               (PanelPluginExternal              *external,
-                                                                   XfcePanelPluginProviderPropType   type,
-                                                                   const GValue                     *value);
-static void         panel_plugin_external_queue_add_action        (PanelPluginExternal              *external,
-                                                                   XfcePanelPluginProviderPropType   type);
 static const gchar *panel_plugin_external_get_name                (XfcePanelPluginProvider          *provider);
 static gint         panel_plugin_external_get_unique_id           (XfcePanelPluginProvider          *provider);
 static void         panel_plugin_external_set_size                (XfcePanelPluginProvider          *provider,
@@ -131,13 +121,8 @@ struct _PanelPluginExternalPrivate
 
   /* delayed spawning */
   guint       spawn_timeout_id;
-
-  /* silence allocation warnings */
-  guint       resize_timeout_id;
 };
 
-gulong global_resize_timeout_id = 0;
-
 enum
 {
   PROP_0,
@@ -148,7 +133,7 @@ enum
 
 
 
-G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PanelPluginExternal, panel_plugin_external, GTK_TYPE_SOCKET,
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PanelPluginExternal, panel_plugin_external, GTK_TYPE_BOX,
                                   G_ADD_PRIVATE (PanelPluginExternal)
                                   G_IMPLEMENT_INTERFACE (XFCE_TYPE_PANEL_PLUGIN_PROVIDER,
                                                          panel_plugin_external_provider_init))
@@ -160,7 +145,6 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
 {
   GObjectClass   *gobject_class;
   GtkWidgetClass *gtkwidget_class;
-  GtkSocketClass *gtksocket_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = panel_plugin_external_finalize;
@@ -168,14 +152,9 @@ panel_plugin_external_class_init (PanelPluginExternalClass *klass)
   gobject_class->get_property = panel_plugin_external_get_property;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
-  gtkwidget_class->size_allocate = panel_plugin_external_size_allocate;
   gtkwidget_class->realize = panel_plugin_external_realize;
   gtkwidget_class->unrealize = panel_plugin_external_unrealize;
 
-  gtksocket_class = GTK_SOCKET_CLASS (klass);
-  gtksocket_class->plug_added = panel_plugin_external_plug_added;
-  gtksocket_class->plug_removed = panel_plugin_external_plug_removed;
-
   g_object_class_install_property (gobject_class,
                                    PROP_UNIQUE_ID,
                                    g_param_spec_int ("unique-id",
@@ -219,7 +198,6 @@ panel_plugin_external_init (PanelPluginExternal *external)
   external->priv->embedded = FALSE;
   external->priv->pid = 0;
   external->priv->spawn_timeout_id = 0;
-  external->priv->resize_timeout_id = 0;
 
   /* signal to pass gtk_widget_set_sensitive() changes to the remote window */
   g_signal_connect (G_OBJECT (external), "notify::sensitive",
@@ -265,12 +243,6 @@ panel_plugin_external_finalize (GObject *object)
   if (external->priv->spawn_timeout_id != 0)
     g_source_remove (external->priv->spawn_timeout_id);
 
-  if (external->priv->resize_timeout_id != 0)
-    {
-      g_source_remove (external->priv->resize_timeout_id);
-      global_resize_timeout_id -= external->priv->resize_timeout_id;
-    }
-
   if (external->priv->watch_id != 0)
     {
       /* remove the child watch and don't leave zombies */
@@ -356,67 +328,12 @@ panel_plugin_external_set_property (GObject      *object,
 
 
 
-static void
-panel_plugin_external_queue_resize (GtkWidget *widget,
-                                    gpointer   data)
-{
-  if (PANEL_IS_PLUGIN_EXTERNAL (data))
-    gtk_widget_queue_resize (data);
-}
-
-
-
-static gboolean
-panel_plugin_external_queue_resize_timeout (gpointer data)
-{
-  PanelPluginExternal *external = data;
-
-  if (! gdk_window_is_visible (gtk_socket_get_plug_window (data)))
-    return TRUE;
-
-  global_resize_timeout_id -= external->priv->resize_timeout_id;
-  external->priv->resize_timeout_id = 0;
-  if (global_resize_timeout_id == 0)
-    gtk_container_foreach (GTK_CONTAINER (gtk_widget_get_parent (data)),
-                           panel_plugin_external_queue_resize, data);
-
-  return FALSE;
-}
-
-
-
-static void
-panel_plugin_external_size_allocate (GtkWidget     *widget,
-                                     GtkAllocation *allocation)
-{
-  PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (widget);
-
-  if (global_resize_timeout_id != 0)
-    {
-      if (external->priv->resize_timeout_id != 0)
-        {
-          global_resize_timeout_id -= external->priv->resize_timeout_id;
-          g_source_remove (external->priv->resize_timeout_id);
-          external->priv->resize_timeout_id =
-            g_timeout_add_seconds (1, panel_plugin_external_queue_resize_timeout, external);
-          global_resize_timeout_id += external->priv->resize_timeout_id;
-        }
-
-      allocation->width = MAX (allocation->width, 16);
-      allocation->height = MAX (allocation->height, 16);
-    }
-
-  GTK_WIDGET_CLASS (panel_plugin_external_parent_class)->size_allocate (widget, allocation);
-}
-
-
-
 static void
 panel_plugin_external_realize (GtkWidget *widget)
 {
   PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (widget);
 
-  /* realize the socket first */
+  /* realize the widget first */
   (*GTK_WIDGET_CLASS (panel_plugin_external_parent_class)->realize) (widget);
 
   if (external->priv->pid == 0)
@@ -461,45 +378,6 @@ panel_plugin_external_unrealize (GtkWidget *widget)
 
 
 
-static void
-panel_plugin_external_plug_added (GtkSocket *socket)
-{
-  PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (socket);
-
-  external->priv->embedded = TRUE;
-  external->priv->resize_timeout_id =
-    g_timeout_add_seconds (1, panel_plugin_external_queue_resize_timeout, external);
-  global_resize_timeout_id += external->priv->resize_timeout_id;
-
-  panel_debug (PANEL_DEBUG_EXTERNAL,
-               "%s-%d: child is embedded; %d properties in queue",
-               panel_module_get_name (external->module),
-               external->unique_id,
-               g_slist_length (external->priv->queue));
-
-  /* send queue to wrapper */
-  panel_plugin_external_queue_send_to_child (external);
-}
-
-
-
-static gboolean
-panel_plugin_external_plug_removed (GtkSocket *socket)
-{
-  PanelPluginExternal *external = PANEL_PLUGIN_EXTERNAL (socket);
-
-  external->priv->embedded = FALSE;
-
-  panel_debug (PANEL_DEBUG_EXTERNAL,
-               "%s-%d: child is unembedded",
-               panel_module_get_name (external->module),
-               external->unique_id);
-
-  return TRUE;
-}
-
-
-
 static gboolean
 panel_plugin_external_child_ask_restart_dialog (GtkWindow   *parent,
                                                 const gchar *plugin_name)
@@ -534,16 +412,6 @@ panel_plugin_external_child_ask_restart_dialog (GtkWindow   *parent,
 
 
 
-static gboolean
-panel_plugin_external_remove (gpointer data)
-{
-  /* cleanup the plugin configuration (in PanelApplication) and destroy 'external' */
-  xfce_panel_plugin_provider_emit_signal (data, PROVIDER_SIGNAL_REMOVE_PLUGIN);
-  return FALSE;
-}
-
-
-
 static gboolean
 panel_plugin_external_child_ask_restart (PanelPluginExternal *external)
 {
@@ -575,9 +443,10 @@ panel_plugin_external_child_ask_restart (PanelPluginExternal *external)
                                NULL);
         }
 
-      /* delay this until we get out of any other idle func, as this triggers the
-       * finalization of 'external' */
-      g_idle_add_full (G_PRIORITY_HIGH, panel_plugin_external_remove, external, NULL);
+      /* cleanup the plugin configuration (in PanelApplication) and
+       * destroy the plugin */
+      xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (external),
+                                              PROVIDER_SIGNAL_REMOVE_PLUGIN);
 
       return FALSE;
     }
@@ -807,7 +676,7 @@ panel_plugin_external_child_watch (GPid     pid,
 
   /* reset the pid, it can't be embedded as well */
   external->priv->pid = 0;
-  external->priv->embedded = FALSE;
+  panel_plugin_external_set_embedded (external, FALSE);
 
   panel_debug (PANEL_DEBUG_EXTERNAL,
                "%s-%d: child exited with status %d",
@@ -840,9 +709,9 @@ panel_plugin_external_child_watch (GPid     pid,
                      panel_module_get_name (external->module),
                      external->unique_id, WEXITSTATUS (status));
 
-          /* delay this until we get out of any other idle func, as this triggers the
-           * finalization of 'external' */
-          g_idle_add_full (G_PRIORITY_HIGH, panel_plugin_external_remove, external, NULL);
+          /* cleanup the plugin configuration (in PanelApplication) */
+          xfce_panel_plugin_provider_emit_signal (XFCE_PANEL_PLUGIN_PROVIDER (external),
+                                                  PROVIDER_SIGNAL_REMOVE_PLUGIN);
 
           goto close_pid;
         }
@@ -873,7 +742,8 @@ close_pid:
 static void
 panel_plugin_external_child_watch_destroyed (gpointer user_data)
 {
-  PANEL_PLUGIN_EXTERNAL (user_data)->priv->watch_id = 0;
+  if (PANEL_IS_PLUGIN_EXTERNAL (user_data))
+    PANEL_PLUGIN_EXTERNAL (user_data)->priv->watch_id = 0;
 }
 
 
@@ -916,45 +786,6 @@ panel_plugin_external_queue_send_to_child (PanelPluginExternal *external)
 
 
 
-static void
-panel_plugin_external_queue_add (PanelPluginExternal             *external,
-                                 XfcePanelPluginProviderPropType  type,
-                                 const GValue                    *value)
-{
-  PluginProperty *prop;
-
-  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
-  panel_return_if_fail (G_TYPE_CHECK_VALUE (value));
-
-  prop = g_slice_new0 (PluginProperty);
-  prop->type = type;
-  g_value_init (&prop->value, G_VALUE_TYPE (value));
-  g_value_copy (value, &prop->value);
-
-  external->priv->queue = g_slist_prepend (external->priv->queue, prop);
-
-  if (external->priv->embedded)
-    panel_plugin_external_queue_send_to_child (external);
-}
-
-
-
-static void
-panel_plugin_external_queue_add_action (PanelPluginExternal             *external,
-                                        XfcePanelPluginProviderPropType  type)
-{
-  GValue value = { 0, };
-
-  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
-
-  /* add to queue with noop boolean */
-  g_value_init (&value, G_TYPE_BOOLEAN);
-  panel_plugin_external_queue_add (external, type, &value);
-  g_value_unset (&value);
-}
-
-
-
 static const gchar *
 panel_plugin_external_get_name (XfcePanelPluginProvider *provider)
 {
@@ -1229,6 +1060,45 @@ panel_plugin_external_set_sensitive (PanelPluginExternal *external)
 
 
 
+void
+panel_plugin_external_queue_add (PanelPluginExternal             *external,
+                                 XfcePanelPluginProviderPropType  type,
+                                 const GValue                    *value)
+{
+  PluginProperty *prop;
+
+  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
+  panel_return_if_fail (G_TYPE_CHECK_VALUE (value));
+
+  prop = g_slice_new0 (PluginProperty);
+  prop->type = type;
+  g_value_init (&prop->value, G_VALUE_TYPE (value));
+  g_value_copy (value, &prop->value);
+
+  external->priv->queue = g_slist_prepend (external->priv->queue, prop);
+
+  if (external->priv->embedded)
+    panel_plugin_external_queue_send_to_child (external);
+}
+
+
+
+void
+panel_plugin_external_queue_add_action (PanelPluginExternal             *external,
+                                        XfcePanelPluginProviderPropType  type)
+{
+  GValue value = { 0, };
+
+  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
+
+  /* add to queue with noop boolean */
+  g_value_init (&value, G_TYPE_BOOLEAN);
+  panel_plugin_external_queue_add (external, type, &value);
+  g_value_unset (&value);
+}
+
+
+
 void
 panel_plugin_external_restart (PanelPluginExternal *external)
 {
@@ -1276,26 +1146,9 @@ void
 panel_plugin_external_set_background_color (PanelPluginExternal *external,
                                             const GdkRGBA       *color)
 {
-  GValue value = { 0, };
-
   panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
 
-  if (G_LIKELY (color != NULL))
-    {
-      g_value_init (&value, G_TYPE_STRING);
-      g_value_take_string (&value, gdk_rgba_to_string (color));
-
-      panel_plugin_external_queue_add (external,
-                                       PROVIDER_PROP_TYPE_SET_BACKGROUND_COLOR,
-                                       &value);
-
-      g_value_unset (&value);
-    }
-  else
-    {
-      panel_plugin_external_queue_add_action (external,
-                                              PROVIDER_PROP_TYPE_ACTION_BACKGROUND_UNSET);
-    }
+  PANEL_PLUGIN_EXTERNAL_GET_CLASS (external)->set_background_color (external, color);
 }
 
 
@@ -1304,25 +1157,59 @@ void
 panel_plugin_external_set_background_image (PanelPluginExternal *external,
                                             const gchar         *image)
 {
-  GValue value = { 0, };
-
   panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
 
-  if (G_UNLIKELY (image != NULL))
+  PANEL_PLUGIN_EXTERNAL_GET_CLASS (external)->set_background_image (external, image);
+}
+
+
+
+void
+panel_plugin_external_set_geometry (PanelPluginExternal *external,
+                                    PanelWindow *window)
+{
+  panel_return_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external));
+
+  PANEL_PLUGIN_EXTERNAL_GET_CLASS (external)->set_geometry (external, window);
+}
+
+
+
+gboolean
+panel_plugin_external_pointer_is_outside (PanelPluginExternal *external)
+{
+  panel_return_val_if_fail (PANEL_IS_PLUGIN_EXTERNAL (external), FALSE);
+
+  return PANEL_PLUGIN_EXTERNAL_GET_CLASS (external)->pointer_is_outside (external);
+}
+
+
+
+void
+panel_plugin_external_set_embedded (PanelPluginExternal *external,
+                                    gboolean embedded)
+{
+  if (external->priv->embedded == !!embedded)
+    return;
+
+  external->priv->embedded = embedded;
+  if (embedded)
     {
-      g_value_init (&value, G_TYPE_STRING);
-      g_value_set_string (&value, image);
+      panel_debug (PANEL_DEBUG_EXTERNAL,
+                   "%s-%d: child is embedded; %d properties in queue",
+                   panel_module_get_name (external->module),
+                   external->unique_id,
+                   g_slist_length (external->priv->queue));
 
-      panel_plugin_external_queue_add (external,
-                                       PROVIDER_PROP_TYPE_SET_BACKGROUND_IMAGE,
-                                       &value);
-
-      g_value_unset (&value);
+      /* send queue to wrapper */
+      panel_plugin_external_queue_send_to_child (external);
     }
   else
     {
-      panel_plugin_external_queue_add_action (external,
-                                              PROVIDER_PROP_TYPE_ACTION_BACKGROUND_UNSET);
+      panel_debug (PANEL_DEBUG_EXTERNAL,
+                   "%s-%d: child is unembedded",
+                   panel_module_get_name (external->module),
+                   external->unique_id);
     }
 }
 
