@@ -210,6 +210,7 @@ panel_preferences_dialog_init (PanelPreferencesDialog *dialog)
   panel_application_windows_blocked (dialog->application, TRUE);
 
   /* load the builder data into the object */
+  gtk_builder_set_translation_domain (GTK_BUILDER (dialog), GETTEXT_PACKAGE);
   gtk_builder_add_from_string (GTK_BUILDER (dialog), panel_preferences_dialog_ui,
                                panel_preferences_dialog_ui_length, NULL);
 
@@ -493,7 +494,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   panel_preferences_dialog_bindings_add (dialog, "span-monitors", "active", 0);
   panel_preferences_dialog_bindings_add (dialog, "position-locked", "active", 0);
   panel_preferences_dialog_bindings_add (dialog, "autohide-behavior", "active", 0);
-  panel_preferences_dialog_bindings_add (dialog, "enable-struts", "active", 0);
+  panel_preferences_dialog_bindings_add (dialog, "disable-struts", "active", 0);
   panel_preferences_dialog_bindings_add (dialog, "size", "value", 0);
   panel_preferences_dialog_bindings_add (dialog, "nrows", "value", 0);
   panel_preferences_dialog_bindings_add (dialog, "length-adjust", "active", 0);
@@ -539,12 +540,12 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
 
   g_object_get (G_OBJECT (dialog->active), "output-name", &output_name, NULL);
 
-  if (n_monitors > 1 || !panel_str_is_empty (output_name))
+  if (n_monitors > 1 || !xfce_str_is_empty (output_name))
     {
       gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                          OUTPUT_NAME, "Automatic",
                                          OUTPUT_TITLE, _("Automatic"), -1);
-      if (panel_str_is_empty (output_name) ||
+      if (xfce_str_is_empty (output_name) ||
           g_strcmp0 (output_name, "Automatic") == 0)
         {
           gtk_combo_box_set_active_iter (GTK_COMBO_BOX (object), &iter);
@@ -567,7 +568,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
             {
               monitor = gdk_display_get_monitor(display, i);
               name = g_strdup(gdk_monitor_get_model (monitor));
-              if (panel_str_is_empty (name))
+              if (xfce_str_is_empty (name))
                 {
                   g_free (name);
 
@@ -597,7 +598,7 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
         }
 
       /* add the output from the config if still nothing has been selected */
-      if (!output_selected && !panel_str_is_empty (output_name))
+      if (!output_selected && !xfce_str_is_empty (output_name))
         {
           gtk_list_store_insert_with_values (GTK_LIST_STORE (store), &iter, n++,
                                              OUTPUT_NAME, output_name,
@@ -627,7 +628,8 @@ panel_preferences_dialog_bindings_update (PanelPreferencesDialog *dialog)
   object = gtk_builder_get_object (GTK_BUILDER (dialog), "span-monitors");
   panel_return_if_fail (GTK_IS_WIDGET (object));
   gtk_widget_set_sensitive (GTK_WIDGET (object), span_monitors_sensitive);
-  g_object_set (G_OBJECT (object), "visible", n_monitors > 1, NULL);
+  g_object_set (G_OBJECT (object), "visible",
+                n_monitors > 1 && GDK_IS_X11_DISPLAY (display), NULL);
 
   g_free (output_name);
 
@@ -700,7 +702,7 @@ panel_preferences_dialog_autohide_changed (GtkComboBox            *combobox,
   panel_return_if_fail (PANEL_IS_PREFERENCES_DIALOG (dialog));
   panel_return_if_fail (PANEL_WINDOW (dialog->active));
 
-  object = gtk_builder_get_object (GTK_BUILDER (dialog), "enable-struts");
+  object = gtk_builder_get_object (GTK_BUILDER (dialog), "disable-struts");
   panel_return_if_fail (GTK_IS_WIDGET (object));
 
   /* make "don't reserve space on borders" sensitive only when autohide is disabled */
@@ -1615,6 +1617,7 @@ panel_preferences_dialog_item_selection_changed (GtkTreeSelection       *selecti
 
 
 
+#ifdef HAVE_GTK_X11
 static void
 panel_preferences_dialog_plug_deleted (GtkWidget *plug)
 {
@@ -1623,6 +1626,7 @@ panel_preferences_dialog_plug_deleted (GtkWidget *plug)
 
   g_object_unref (G_OBJECT (dialog_singleton));
 }
+#endif
 
 
 
@@ -1631,12 +1635,14 @@ panel_preferences_dialog_show_internal (PanelWindow *active,
                                         Window       socket_window)
 {
   gint         panel_id = 0;
-  GObject     *window, *combo;
+  GObject     *window, *widget;
   GdkScreen   *screen;
   GSList      *windows;
+#ifdef HAVE_GTK_X11
   GtkWidget   *plug;
   GObject     *plug_child;
   GtkWidget   *content_area;
+#endif
 
   panel_return_if_fail (active == NULL || PANEL_IS_WINDOW (active));
 
@@ -1660,14 +1666,33 @@ panel_preferences_dialog_show_internal (PanelWindow *active,
     }
 
   /* select the active window in the dialog */
-  combo = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "panel-combobox");
-  panel_return_if_fail (GTK_IS_WIDGET (combo));
+  widget = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "panel-combobox");
+  panel_return_if_fail (GTK_IS_WIDGET (widget));
   panel_id = panel_window_get_id (active);
   if (!panel_preferences_dialog_panel_combobox_rebuild (dialog_singleton, panel_id))
-    gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
+    gtk_combo_box_set_active (GTK_COMBO_BOX (widget), 0);
+
+  /* select item in the dialog if needed */
+  if (GDK_IS_WAYLAND_DISPLAY (gdk_display_get_default ()))
+    {
+      const gchar *item = g_object_get_data (G_OBJECT (active), "prefs-dialog-item");
+      if (item != NULL)
+        {
+          GtkTreePath *path = gtk_tree_path_new_from_string (item);
+          widget = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "notebook");
+          panel_return_if_fail (GTK_IS_WIDGET (widget));
+          gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), 2);
+          widget = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "item-treeview");
+          panel_return_if_fail (GTK_IS_WIDGET (widget));
+          gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
+          gtk_tree_path_free (path);
+          g_object_set_data (G_OBJECT (active), "prefs-dialog-item", NULL);
+        }
+    }
 
   window = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "dialog");
   panel_return_if_fail (GTK_IS_WIDGET (window));
+#ifdef HAVE_GTK_X11
   plug_child = gtk_builder_get_object (GTK_BUILDER (dialog_singleton), "plug-child");
   panel_return_if_fail (GTK_IS_WIDGET (plug_child));
 
@@ -1689,6 +1714,7 @@ panel_preferences_dialog_show_internal (PanelWindow *active,
           G_CALLBACK (panel_preferences_dialog_plug_deleted), NULL);
       gtk_widget_destroy (plug);
     }
+#endif
 
   if (socket_window == 0)
     {
@@ -1702,6 +1728,7 @@ panel_preferences_dialog_show_internal (PanelWindow *active,
       gtk_window_present (GTK_WINDOW (window));
       panel_application_take_dialog (dialog_singleton->application, GTK_WINDOW (window));
     }
+#ifdef HAVE_GTK_X11
   else
     {
       /* hide window */
@@ -1718,6 +1745,7 @@ panel_preferences_dialog_show_internal (PanelWindow *active,
       xfce_widget_reparent (GTK_WIDGET (plug_child), plug);
       gtk_widget_show (GTK_WIDGET (plug_child));
     }
+#endif
 }
 
 
