@@ -111,7 +111,7 @@ static gboolean  applications_menu_plugin_remote_event         (XfcePanelPlugin
 static gboolean  applications_menu_plugin_menu                 (GtkWidget              *button,
                                                                 GdkEventButton         *event,
                                                                 ApplicationsMenuPlugin *plugin);
-static void      applications_menu_plugin_menu_selection_done  (GtkMenuShell           *menu,
+static void      applications_menu_plugin_menu_popdown         (GtkMenuShell           *menu,
                                                                 ApplicationsMenuPlugin *plugin);
 static void      applications_menu_plugin_set_garcon_menu      (ApplicationsMenuPlugin *plugin);
 static void      applications_menu_button_theme_changed        (ApplicationsMenuPlugin *plugin);
@@ -247,11 +247,8 @@ applications_menu_plugin_init (ApplicationsMenuPlugin *plugin)
   plugin->show_button_title = TRUE;
   gtk_widget_show (plugin->label);
 
-  /* prepare the menu: some situations, such as cancelling a dnd, require connecting to
-   * "selection-done" instead of "deactivate" */
+  /* prepare the menu */
   plugin->menu = garcon_gtk_menu_new (NULL);
-  g_signal_connect (G_OBJECT (plugin->menu), "selection-done",
-      G_CALLBACK (applications_menu_plugin_menu_selection_done), plugin);
 
   plugin->style_updated_id = g_signal_connect_swapped (G_OBJECT (plugin->button), "style-updated",
                                                        G_CALLBACK (applications_menu_button_theme_changed), plugin);
@@ -374,7 +371,7 @@ applications_menu_plugin_set_property (GObject      *object,
       gtk_label_set_text (GTK_LABEL (plugin->label),
           plugin->button_title != NULL ? plugin->button_title : "");
       gtk_widget_set_tooltip_text (plugin->button,
-          panel_str_is_empty (plugin->button_title) ? NULL : plugin->button_title);
+          xfce_str_is_empty (plugin->button_title) ? NULL : plugin->button_title);
 
       /* check if the label still fits */
       if (xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin)) == XFCE_PANEL_PLUGIN_MODE_DESKBAR
@@ -387,8 +384,8 @@ applications_menu_plugin_set_property (GObject      *object,
     case PROP_BUTTON_ICON:
       g_free (plugin->button_icon);
       plugin->button_icon =
-        panel_str_is_empty (g_value_get_string (value)) ? g_strdup (DEFAULT_ICON_NAME)
-                                                        : g_value_dup_string (value);
+        xfce_str_is_empty (g_value_get_string (value)) ? g_strdup (DEFAULT_ICON_NAME)
+                                                       : g_value_dup_string (value);
 
       force_a_resize = TRUE;
       break;
@@ -735,7 +732,7 @@ applications_menu_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
 
   object = gtk_builder_get_object (builder, "custom-file");
   panel_return_if_fail (GTK_IS_FILE_CHOOSER_BUTTON (object));
-  if (!panel_str_is_empty (plugin->custom_menu_file))
+  if (!xfce_str_is_empty (plugin->custom_menu_file))
     gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (object), plugin->custom_menu_file);
   g_signal_connect (G_OBJECT (object), "file-set",
      G_CALLBACK (applications_menu_plugin_configure_plugin_file_set), plugin);
@@ -751,7 +748,6 @@ applications_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
                                        const GValue    *value)
 {
   ApplicationsMenuPlugin *plugin = XFCE_APPLICATIONS_MENU_PLUGIN (panel_plugin);
-  GtkWidget *invisible;
 
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
 
@@ -760,20 +756,14 @@ applications_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
       || ! gtk_widget_get_visible (GTK_WIDGET (panel_plugin)))
     return FALSE;
 
-  invisible = gtk_invisible_new ();
-  gtk_widget_show (invisible);
-
   /* a menu is already shown, don't popup another one */
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (plugin->button))
-      || ! panel_utils_device_grab (invisible))
-    {
-      gtk_widget_destroy (invisible);
-      return TRUE;
-    }
+      || ! panel_utils_device_grab (plugin->button))
+    return TRUE;
 
   /*
-   * The menu will take over the grab when it is shown or it will be lost when destroying
-   * invisible below. This way we are sure that other invocations of the command by
+   * The menu will take over the grab when it is shown, and in the rare cases that it is not,
+   * this is not a big deal. This way we are sure that other invocations of the command by
    * keyboard shortcut will not interfere.
    */
   if (value != NULL
@@ -789,8 +779,6 @@ applications_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
       applications_menu_plugin_menu (plugin->button, NULL, plugin);
     }
 
-  gtk_widget_destroy (invisible);
-
   /* don't popup another menu */
   return TRUE;
 }
@@ -798,12 +786,14 @@ applications_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
 
 
 static void
-applications_menu_plugin_menu_selection_done (GtkMenuShell           *menu,
-                                              ApplicationsMenuPlugin *plugin)
+applications_menu_plugin_menu_popdown (GtkMenuShell           *menu,
+                                       ApplicationsMenuPlugin *plugin)
 {
   panel_return_if_fail (plugin->button == NULL || GTK_IS_TOGGLE_BUTTON (plugin->button));
   panel_return_if_fail (GTK_IS_MENU (menu));
 
+  g_signal_handlers_disconnect_by_func (menu, applications_menu_plugin_menu_popdown, plugin);
+
   /* button is NULL when we popup the menu under the cursor position */
   if (plugin->button != NULL)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), FALSE);
@@ -856,8 +846,6 @@ applications_menu_plugin_menu (GtkWidget              *button,
                                GdkEventButton         *event,
                                ApplicationsMenuPlugin *plugin)
 {
-  GdkEvent *free_event = NULL;
-
   panel_return_val_if_fail (XFCE_IS_APPLICATIONS_MENU_PLUGIN (plugin), FALSE);
   panel_return_val_if_fail (button == NULL || plugin->button == button, FALSE);
 
@@ -874,22 +862,20 @@ applications_menu_plugin_menu (GtkWidget              *button,
    * gtk_menu_popup_at_pointer/rect can extract a location correctly from a GdkWindow */
   if (event == NULL)
     {
-      GdkSeat *seat = gdk_display_get_default_seat (gdk_display_get_default ());
-      free_event = gdk_event_new (GDK_BUTTON_PRESS);
-      free_event->button.window = g_object_ref (gdk_get_default_root_window ());
-      gdk_event_set_device (free_event, gdk_seat_get_pointer (seat));
-      event = (GdkEventButton *)free_event;
+      event = g_slice_new0 (GdkEventButton);
+      event->type = GDK_BUTTON_PRESS;
+      event->window = gdk_get_default_root_window ();
     }
 
-  /* do not block panel autohide if popup-command at pointer */
-  if (button == NULL)
-    gtk_menu_popup_at_pointer (GTK_MENU (plugin->menu), (GdkEvent *) event);
-  else
-    xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (plugin->menu),
-                                  button, (GdkEvent *) event);
+  /* when cancelling a dnd only "selection-done" is emitted, whereas "deactivate" is more
+   * generic and it is the only signal emitted on Wayland when clicking outside the menu */
+  g_signal_connect (G_OBJECT (plugin->menu), "deactivate",
+      G_CALLBACK (applications_menu_plugin_menu_popdown), plugin);
+  g_signal_connect (G_OBJECT (plugin->menu), "selection-done",
+      G_CALLBACK (applications_menu_plugin_menu_popdown), plugin);
 
-  if (free_event != NULL)
-    gdk_event_free (free_event);
+  xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (plugin->menu),
+                                button, (GdkEvent *) event);
 
   return TRUE;
 }
