@@ -22,6 +22,7 @@
 
 #include <exo/exo.h>
 #include <gio/gio.h>
+#include <gio/gdesktopappinfo.h>
 #include <libxfce4ui/libxfce4ui.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/libxfce4panel.h>
@@ -29,10 +30,6 @@
 #include <common/panel-utils.h>
 #include <common/panel-private.h>
 
-#ifdef HAVE_GIO_UNIX
-#include <gio/gdesktopappinfo.h>
-#endif
-
 #include "directorymenu.h"
 #include "directorymenu-dialog_ui.h"
 
@@ -253,7 +250,7 @@ directory_menu_plugin_get_property (GObject    *object,
       break;
 
     case PROP_FILE_PATTERN:
-      g_value_set_string (value, panel_str_is_empty (plugin->file_pattern) ?
+      g_value_set_string (value, xfce_str_is_empty (plugin->file_pattern) ?
           "" : plugin->file_pattern);
       break;
 
@@ -286,7 +283,7 @@ directory_menu_plugin_set_property (GObject      *object,
     {
     case PROP_BASE_DIRECTORY:
       path = g_value_get_string (value);
-      if (panel_str_is_empty (path))
+      if (xfce_str_is_empty (path))
         path = g_get_home_dir ();
 
       if (plugin->base_directory != NULL)
@@ -304,8 +301,8 @@ directory_menu_plugin_set_property (GObject      *object,
     case PROP_ICON_NAME:
       g_free (plugin->icon_name);
       plugin->icon_name =
-        panel_str_is_empty (g_value_get_string (value)) ? g_strdup (DEFAULT_ICON_NAME)
-                                                        : g_value_dup_string (value);
+        xfce_str_is_empty (g_value_get_string (value)) ? g_strdup (DEFAULT_ICON_NAME)
+                                                       : g_value_dup_string (value);
       size = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
       directory_menu_plugin_size_changed (XFCE_PANEL_PLUGIN (plugin), size);
       break;
@@ -336,7 +333,7 @@ directory_menu_plugin_set_property (GObject      *object,
       if (G_LIKELY (array != NULL))
         {
           for (i = 0; array[i] != NULL; i++)
-            if (!panel_str_is_empty (array[i]))
+            if (!xfce_str_is_empty (array[i]))
                 plugin->patterns = g_slist_prepend (plugin->patterns,
                     g_pattern_spec_new (array[i]));
 
@@ -411,8 +408,7 @@ directory_menu_plugin_free_data (XfcePanelPlugin *panel_plugin)
 {
   DirectoryMenuPlugin *plugin = XFCE_DIRECTORY_MENU_PLUGIN (panel_plugin);
 
-  if (plugin->base_directory != NULL)
-    g_object_unref (G_OBJECT (plugin->base_directory));
+  g_object_unref (G_OBJECT (plugin->base_directory));
   g_free (plugin->icon_name);
   g_free (plugin->file_pattern);
 
@@ -577,7 +573,6 @@ directory_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
                                     const GValue    *value)
 {
   DirectoryMenuPlugin *plugin = XFCE_DIRECTORY_MENU_PLUGIN (panel_plugin);
-  GtkWidget *invisible;
 
   panel_return_val_if_fail (value == NULL || G_IS_VALUE (value), FALSE);
 
@@ -586,20 +581,14 @@ directory_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
       || ! gtk_widget_get_visible (GTK_WIDGET (panel_plugin)))
     return FALSE;
 
-  invisible = gtk_invisible_new ();
-  gtk_widget_show (invisible);
-
   /* a menu is already shown, don't popup another one */
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (plugin->button))
-      || ! panel_utils_device_grab (invisible))
-    {
-      gtk_widget_destroy (invisible);
-      return TRUE;
-    }
+      || ! panel_utils_device_grab (plugin->button))
+    return TRUE;
 
   /*
-   * The menu will take over the grab when it is shown or it will be lost when destroying
-   * invisible below. This way we are sure that other invocations of the command by
+   * The menu will take over the grab when it is shown, and in the rare cases that it is not,
+   * this is not a big deal. This way we are sure that other invocations of the command by
    * keyboard shortcut will not interfere.
    */
   if (value != NULL
@@ -615,8 +604,6 @@ directory_menu_plugin_remote_event (XfcePanelPlugin *panel_plugin,
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->button), TRUE);
     }
 
-  gtk_widget_destroy (invisible);
-
   /* don't popup another menu */
   return TRUE;
 }
@@ -679,7 +666,6 @@ directory_menu_plugin_menu_sort (gconstpointer a,
 
 
 
-#ifdef HAVE_GIO_UNIX
 static void
 directory_menu_plugin_menu_launch_desktop_file (GtkWidget *mi,
                                                 GAppInfo  *info)
@@ -709,7 +695,6 @@ directory_menu_plugin_menu_launch_desktop_file (GtkWidget *mi,
 
   g_object_unref (G_OBJECT (context));
 }
-#endif
 
 
 
@@ -1067,10 +1052,8 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
   GFile           *dir;
   gboolean         visible;
   GFileType        file_type;
-#ifdef HAVE_GIO_UNIX
   GDesktopAppInfo *desktopinfo;
   const gchar     *description;
-#endif
 
   panel_return_if_fail (XFCE_IS_DIRECTORY_MENU_PLUGIN (plugin));
   panel_return_if_fail (GTK_IS_MENU (menu));
@@ -1167,7 +1150,7 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
               display_name = g_file_info_get_display_name (info);
               if (G_LIKELY (display_name != NULL))
                 for (li = plugin->patterns; !visible && li != NULL; li = li->next)
-                   if (g_pattern_match_string (li->data, display_name))
+                   if (g_pattern_spec_match_string (li->data, display_name))
                      visible = TRUE;
 
               if (!visible)
@@ -1205,7 +1188,6 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
           file = g_file_get_child (dir, g_file_info_get_name (info));
           icon = NULL;
 
-#ifdef HAVE_GIO_UNIX
           /* for native desktop files we make an exception and try
            * to load them like a normal menu */
           desktopinfo = NULL;
@@ -1220,7 +1202,7 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
                   icon = g_app_info_get_icon (G_APP_INFO (desktopinfo));
 
                   /* ignore invalid or hidden files */
-                  if (panel_str_is_empty (display_name)
+                  if (xfce_str_is_empty (display_name)
                       || g_desktop_app_info_get_is_hidden (desktopinfo))
                     {
                       g_object_unref (G_OBJECT (desktopinfo));
@@ -1230,7 +1212,6 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
                     }
                 }
             }
-#endif
 
           mi = panel_image_menu_item_new_with_label (display_name);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
@@ -1257,11 +1238,10 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
               g_signal_connect_after (G_OBJECT (submenu), "hide",
                   G_CALLBACK (directory_menu_plugin_menu_unload), NULL);
             }
-#ifdef HAVE_GIO_UNIX
           else if (G_UNLIKELY (desktopinfo != NULL))
             {
               description = g_app_info_get_description (G_APP_INFO (desktopinfo));
-              if (!panel_str_is_empty (description))
+              if (!xfce_str_is_empty (description))
                 gtk_widget_set_tooltip_text (mi, description);
 
               g_signal_connect_data (G_OBJECT (mi), "activate",
@@ -1270,7 +1250,6 @@ directory_menu_plugin_menu_load (GtkWidget           *menu,
 
               g_object_unref (G_OBJECT (file));
             }
-#endif
           else
             {
               g_signal_connect_data (G_OBJECT (mi), "activate",
@@ -1291,8 +1270,8 @@ static void
 directory_menu_plugin_menu (GtkWidget           *button,
                             DirectoryMenuPlugin *plugin)
 {
-  GtkWidget *menu;
-  GdkEvent  *event = NULL;
+  GtkWidget      *menu;
+  GdkEventButton *event = NULL;
 
   panel_return_if_fail (XFCE_IS_DIRECTORY_MENU_PLUGIN (plugin));
   panel_return_if_fail (button == NULL || plugin->button == button);
@@ -1303,13 +1282,11 @@ directory_menu_plugin_menu (GtkWidget           *button,
 
   /* Panel plugin remote events don't send actual GdkEvents, so construct a minimal one so that
    * gtk_menu_popup_at_pointer/rect can extract a location correctly from a GdkWindow */
-  event = gtk_get_current_event ();
-  if (event == NULL)
+  if (gtk_get_current_event () == NULL)
     {
-      GdkSeat *seat = gdk_display_get_default_seat (gdk_display_get_default ());
-      event = gdk_event_new (GDK_BUTTON_PRESS);
-      event->button.window = g_object_ref (gdk_get_default_root_window ());
-      gdk_event_set_device (event, gdk_seat_get_pointer (seat));
+      event = g_slice_new0 (GdkEventButton);
+      event->type = GDK_BUTTON_PRESS;
+      event->window = gdk_get_default_root_window ();
     }
 
   menu = gtk_menu_new ();
@@ -1320,12 +1297,6 @@ directory_menu_plugin_menu (GtkWidget           *button,
                            g_object_ref (plugin->base_directory),
                            g_object_unref);
   directory_menu_plugin_menu_load (menu, plugin);
-
-  /* do not block panel autohide if popup-command at pointer */
-  if (button == NULL)
-    gtk_menu_popup_at_pointer (GTK_MENU (menu), event);
-  else
-    xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (menu), button, event);
-
-  gdk_event_free (event);
+  xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (menu),
+                                button, (GdkEvent *) event);
 }
