@@ -39,7 +39,6 @@
 #include "launcher-dialog_ui.h"
 
 #ifdef GDK_WINDOWING_X11
-#include <gdk/gdkx.h>
 #define LAUNCHER_WIDGET_XID(widget) ((guint) GDK_WINDOW_XID (gdk_screen_get_root_window (gtk_widget_get_screen (GTK_WIDGET (widget)))))
 #else
 #define LAUNCHER_WIDGET_XID(widget) (0)
@@ -131,7 +130,7 @@ launcher_dialog_add_visible_function (GtkTreeModel *model,
 
   /* get the search string from the item */
   text = gtk_entry_get_text (GTK_ENTRY (user_data));
-  if (G_UNLIKELY (panel_str_is_empty (text)))
+  if (G_UNLIKELY (xfce_str_is_empty (text)))
     return TRUE;
 
   /* casefold the search text */
@@ -141,7 +140,7 @@ launcher_dialog_add_visible_function (GtkTreeModel *model,
 
   /* try the pre-build search string first */
   gtk_tree_model_get (model, iter, COL_SEARCH, &string, -1);
-  if (!panel_str_is_empty (string))
+  if (!xfce_str_is_empty (string))
     {
       /* search */
       visible = (strstr (string, text_casefolded) != NULL);
@@ -150,7 +149,7 @@ launcher_dialog_add_visible_function (GtkTreeModel *model,
     {
       /* get the name */
       gtk_tree_model_get (model, iter, COL_NAME, &string, -1);
-      if (!panel_str_is_empty (string))
+      if (!xfce_str_is_empty (string))
         {
           /* escape and casefold the name */
           escaped = g_markup_escape_text (string, -1);
@@ -861,7 +860,7 @@ launcher_dialog_item_button_clicked (GtkWidget            *button,
           if (xfce_dialog_confirm (GTK_WINDOW (toplevel), "edit-delete", _("_Remove"),
                   _("If you delete an item, it will be permanently removed"),
                   _("Are you sure you want to remove \"%s\"?"),
-                  panel_str_is_empty (display_name) ? _("Unnamed item") : display_name))
+                  xfce_str_is_empty (display_name) ? _("Unnamed item") : display_name))
             {
               /* remove the item from the store */
               gtk_list_store_remove (GTK_LIST_STORE (model), &iter_a);
@@ -1043,7 +1042,9 @@ launcher_dialog_items_set_item (GtkTreeModel         *model,
 {
   const gchar *name, *comment;
   gchar       *markup;
-  GIcon       *icon;
+  GdkPixbuf   *icon = NULL;
+  const gchar *icon_name;
+  gint         w, h;
   gchar       *tooltip;
   GFile       *gfile;
 
@@ -1053,12 +1054,19 @@ launcher_dialog_items_set_item (GtkTreeModel         *model,
   name = garcon_menu_item_get_name (item);
   comment = garcon_menu_item_get_comment (item);
 
-  if (!panel_str_is_empty (comment))
+  if (!xfce_str_is_empty (comment))
     markup = g_markup_printf_escaped ("<b>%s</b>\n%s", name, comment);
   else
     markup = g_markup_printf_escaped ("<b>%s</b>", name);
 
-  icon = launcher_plugin_tooltip_icon (garcon_menu_item_get_icon_name (item));
+  icon_name = garcon_menu_item_get_icon_name (item);
+  if (!xfce_str_is_empty (icon_name))
+    {
+      if (!gtk_icon_size_lookup (GTK_ICON_SIZE_DND, &w, &h))
+        w = h = 32;
+
+      icon = xfce_panel_pixbuf_from_source (icon_name, NULL, MIN (w, h));
+    }
 
   if (dialog != NULL)
     g_signal_handlers_block_by_func (G_OBJECT (model),
