@@ -73,6 +73,8 @@ static void               launcher_plugin_screen_position_changed       (XfcePan
                                                                          XfceScreenPosition    position);
 static LauncherArrowType  launcher_plugin_default_arrow_type            (LauncherPlugin       *plugin);
 static void               launcher_plugin_pack_widgets                  (LauncherPlugin       *plugin);
+static cairo_surface_t   *launcher_plugin_tooltip_icon                  (GtkWidget            *widget,
+                                                                         const gchar          *icon_name);
 static void               launcher_plugin_tooltip_icon_invalidate       (GObject              *object);
 static void               launcher_plugin_icon_invalidate               (LauncherPlugin       *plugin);
 static void               launcher_plugin_menu_deactivate               (GtkWidget            *menu,
@@ -366,6 +368,8 @@ launcher_plugin_init (LauncherPlugin *plugin)
   /* invalidate tooltip icon when needed */
   plugin->theme_change_id = g_signal_connect_swapped (gtk_icon_theme_get_default (), "changed",
       G_CALLBACK (launcher_plugin_tooltip_icon_invalidate), plugin->button);
+  g_signal_connect (plugin->button, "notify::scale-factor",
+      G_CALLBACK (launcher_plugin_tooltip_icon_invalidate), NULL);
   g_signal_connect (plugin, "notify::scale-factor",
       G_CALLBACK (launcher_plugin_icon_invalidate), NULL);
   g_signal_connect (plugin, "notify::scale-factor",
@@ -380,7 +384,6 @@ launcher_plugin_init (LauncherPlugin *plugin)
                                   GTK_STYLE_PROVIDER (css_provider),
                                   GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
   g_free (css_string);
-  g_object_unref (css_provider);
 
   plugin->child = gtk_image_new ();
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->child);
@@ -1030,7 +1033,7 @@ launcher_plugin_construct (XfcePanelPlugin *panel_plugin)
     {
       /* get the plugin arguments list */
       uris = xfce_panel_plugin_get_arguments (panel_plugin);
-      if (G_LIKELY (uris != NULL && *uris != NULL))
+      if (G_LIKELY (uris != NULL))
         {
           /* create array with all the uris */
           array = g_ptr_array_new ();
@@ -1413,24 +1416,32 @@ launcher_plugin_pack_widgets (LauncherPlugin *plugin)
 
 
 
-GIcon *
-launcher_plugin_tooltip_icon (const gchar *icon_name)
+cairo_surface_t *
+launcher_plugin_tooltip_icon (GtkWidget *widget,
+                              const gchar *icon_name)
 {
-  GIcon *icon;
+  cairo_surface_t *surface = NULL;
+  GdkPixbuf *pixbuf;
+  gint scale_factor, size;
 
-  if (panel_str_is_empty (icon_name))
+  if (xfce_str_is_empty (icon_name))
     return NULL;
 
+  scale_factor = gtk_widget_get_scale_factor (widget);
+  size = scale_factor * 32;
   if (G_UNLIKELY (g_path_is_absolute (icon_name)))
-    {
-      GFile *file = g_file_new_for_path (icon_name);
-      icon = g_file_icon_new (file);
-      g_object_unref (file);
-    }
+    pixbuf = gdk_pixbuf_new_from_file_at_scale (icon_name, size, size, TRUE, NULL);
   else
-    icon = g_themed_icon_new (icon_name);
+    pixbuf = gtk_icon_theme_load_icon_for_scale (gtk_icon_theme_get_default (), icon_name, 32,
+                                                 scale_factor, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
 
-  return icon;
+  if (pixbuf != NULL)
+    {
+      surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+      g_object_unref (pixbuf);
+    }
+
+  return surface;
 }
 
 
@@ -1597,7 +1608,7 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
       /* create the menu item */
       name = garcon_menu_item_get_name (item);
       mi = gtk_menu_item_new ();
-      label = gtk_label_new (panel_str_is_empty (name) ? _("Unnamed Item") : name);
+      label = gtk_label_new (xfce_str_is_empty (name) ? _("Unnamed Item") : name);
       gtk_label_set_xalign (GTK_LABEL (label), 0.0);
       box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 4);
       gtk_box_pack_end (GTK_BOX (box), label, TRUE, TRUE, 0);
@@ -1623,6 +1634,8 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
           /* invalidate tooltip icon when needed */
           g_signal_connect_object (gtk_icon_theme_get_default (), "changed",
               G_CALLBACK (launcher_plugin_tooltip_icon_invalidate), mi, G_CONNECT_SWAPPED);
+          g_signal_connect (mi, "notify::scale-factor",
+              G_CALLBACK (launcher_plugin_tooltip_icon_invalidate), NULL);
         }
 
       /* depending on the menu position we prepend or append */
@@ -1634,7 +1647,7 @@ launcher_plugin_menu_construct (LauncherPlugin *plugin)
       /* set the icon if one is set */
       icon_name = garcon_menu_item_get_icon_name (item);
 
-      if (panel_str_is_empty (icon_name))
+      if (xfce_str_is_empty (icon_name))
         {
           /* use an empty placeholder icon */
           image = gtk_image_new_from_icon_name ("", GTK_ICON_SIZE_DND);
@@ -1681,7 +1694,6 @@ launcher_plugin_menu_popup (gpointer user_data)
 {
   LauncherPlugin *plugin = XFCE_LAUNCHER_PLUGIN (user_data);
   gint            x, y;
-  GdkEvent       *event = NULL;
 
   panel_return_val_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin), FALSE);
 
@@ -1695,25 +1707,12 @@ launcher_plugin_menu_popup (gpointer user_data)
   else
     gtk_widget_set_state_flags (GTK_WIDGET (plugin->button), GTK_STATE_FLAG_CHECKED, TRUE);
 
-  /* avoid warning when there is no current event */
-  event = gtk_get_current_event ();
-  if (event == NULL)
-    {
-      GdkSeat *seat = gdk_display_get_default_seat (gdk_display_get_default ());
-      event = gdk_event_new (GDK_BUTTON_PRESS);
-      event->button.window = g_object_ref (gdk_get_default_root_window ());
-      gdk_event_set_device (event, gdk_seat_get_pointer (seat));
-    }
-
   /* popup the menu */
   xfce_panel_plugin_popup_menu (XFCE_PANEL_PLUGIN (plugin), GTK_MENU (plugin->menu),
-                                plugin->button, event);
+                                plugin->button, NULL);
 
-  gdk_event_free (event);
-
-  /* fallback to manual positioning, this is used with drag motion over the arrow button
-   * from e.g. Thunar, not from the app menu, where showing the menu does not work for
-   * some other reason: see https://gitlab.xfce.org/xfce/xfce4-panel/-/issues/205 */
+  /* fallback to manual positioning, this is used with
+   * drag motion over the arrow button */
   if (!gtk_widget_get_visible (plugin->menu))
     {
       /* make sure the size is allocated */
@@ -1806,7 +1805,7 @@ launcher_plugin_button_update (LauncherPlugin *plugin)
       panel_return_if_fail (GTK_IS_WIDGET (plugin->child));
 
       icon_name = garcon_menu_item_get_icon_name (item);
-      if (!panel_str_is_empty (icon_name))
+      if (!xfce_str_is_empty (icon_name))
         {
           if (g_path_is_absolute (icon_name))
             {
@@ -1878,6 +1877,14 @@ launcher_plugin_button_update_action_menu (LauncherPlugin *plugin)
   panel_return_if_fail (XFCE_IS_LAUNCHER_PLUGIN (plugin));
   panel_return_if_fail (plugin->menu == NULL);
 
+  /* If there are >1 items in the launcher don't show the action menu */
+  if (LIST_HAS_TWO_OR_MORE_ENTRIES (plugin->items))
+    {
+      xfce_panel_plugin_menu_destroy (XFCE_PANEL_PLUGIN (plugin));
+      plugin->action_menu = NULL;
+      return;
+    }
+
   if (G_LIKELY (plugin->items != NULL))
     item = GARCON_MENU_ITEM (plugin->items->data);
 
@@ -2312,13 +2319,6 @@ launcher_plugin_arrow_drag_leave_timeout (gpointer user_data)
   if (G_UNLIKELY (plugin->menu == NULL))
     return FALSE;
 
-  /* leave if the menu is already hidden */
-  if (! gtk_widget_get_visible (plugin->menu))
-    {
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (plugin->arrow), FALSE);
-      return FALSE;
-    }
-
   device = gdk_seat_get_pointer (gdk_display_get_default_seat (gtk_widget_get_display (menu)));
   if (device == NULL)
     return FALSE;
@@ -2401,40 +2401,48 @@ launcher_plugin_item_query_tooltip (GtkWidget      *widget,
                                     GtkTooltip     *tooltip,
                                     GarconMenuItem *item)
 {
+  GtkWidget *box, *label, *image;
   gchar *markup;
   const gchar *name, *comment;
-  GIcon *icon;
+  cairo_surface_t *surface;
 
   panel_return_val_if_fail (GARCON_IS_MENU_ITEM (item), FALSE);
 
   /* require atleast an item name */
   name = garcon_menu_item_get_name (item);
-  if (panel_str_is_empty (name))
+  if (xfce_str_is_empty (name))
     return FALSE;
 
   comment = garcon_menu_item_get_comment (item);
-  if (!panel_str_is_empty (comment))
+  if (!xfce_str_is_empty (comment))
     {
       markup = g_markup_printf_escaped ("<b>%s</b>\n%s", name, comment);
-      gtk_tooltip_set_markup (tooltip, markup);
+      label = gtk_label_new (markup);
+      gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
       g_free (markup);
     }
   else
     {
-      gtk_tooltip_set_text (tooltip, name);
+      label = gtk_label_new (name);
     }
 
-  icon = g_object_get_data (G_OBJECT (widget), "tooltip-icon");
-  if (G_UNLIKELY (icon == NULL))
+  surface = g_object_get_data (G_OBJECT (widget), "tooltip-icon");
+  if (G_UNLIKELY (surface == NULL))
     {
-      icon = launcher_plugin_tooltip_icon (garcon_menu_item_get_icon_name (item));
-      if (G_LIKELY (icon != NULL))
+      surface = launcher_plugin_tooltip_icon (widget, garcon_menu_item_get_icon_name (item));
+      if (G_LIKELY (surface != NULL))
         {
-          g_object_set_data_full (G_OBJECT (widget), "tooltip-icon", icon, g_object_unref);
+          g_object_set_data_full (G_OBJECT (widget), "tooltip-icon", surface,
+                                  (GDestroyNotify) cairo_surface_destroy);
         }
     }
 
-  gtk_tooltip_set_icon_from_gicon (tooltip, icon, GTK_ICON_SIZE_DND);
+  image = gtk_image_new_from_surface (surface);
+  box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+  gtk_box_pack_start (GTK_BOX (box), image, TRUE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (box), label, TRUE, FALSE, 0);
+  gtk_widget_show_all (box);
+  gtk_tooltip_set_custom (tooltip, box);
 
   return TRUE;
 }
@@ -2458,7 +2466,7 @@ launcher_plugin_item_exec_on_screen (GarconMenuItem *item,
 
   /* get the command */
   command = (gchar*) garcon_menu_item_get_command (item);
-  panel_return_val_if_fail (!panel_str_is_empty (command), FALSE);
+  panel_return_val_if_fail (!xfce_str_is_empty (command), FALSE);
 
   /* expand the field codes */
   icon = garcon_menu_item_get_icon_name (item);
@@ -2511,7 +2519,7 @@ launcher_plugin_item_exec (GarconMenuItem *item,
 
   /* leave when there is nothing to execute */
   command = garcon_menu_item_get_command (item);
-  if (panel_str_is_empty (command))
+  if (xfce_str_is_empty (command))
     return;
 
   if (G_UNLIKELY (uri_list != NULL
@@ -2554,7 +2562,7 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
     text = gtk_clipboard_wait_for_text (clipboard);
 
   /* try the secondary keayboard if the text is empty */
-  if (panel_str_is_empty (text))
+  if (xfce_str_is_empty (text))
     {
       /* get the secondary clipboard text */
       clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
@@ -2562,7 +2570,7 @@ launcher_plugin_item_exec_from_clipboard (GarconMenuItem *item,
         text = gtk_clipboard_wait_for_text (clipboard);
     }
 
-  if (!panel_str_is_empty (text))
+  if (!xfce_str_is_empty (text))
     {
       /* create fake selection data */
       //data.data = (guchar *) text;      //HOWTO?
@@ -2607,7 +2615,7 @@ launcher_plugin_uri_list_extract (GtkSelectionData *data)
       /* create the list of uris */
       for (i = 0; array[i] != NULL; i++)
         {
-          if (!panel_str_is_empty (array[i]))
+          if (!xfce_str_is_empty (array[i]))
             list = g_slist_prepend (list, array[i]);
           else
             g_free (array[i]);
@@ -2626,7 +2634,7 @@ launcher_plugin_uri_list_extract (GtkSelectionData *data)
       for (i = 0; array[i] != NULL; i++)
         {
           /* skip empty strings */
-          if (!!panel_str_is_empty (array[i]))
+          if (!!xfce_str_is_empty (array[i]))
             continue;
 
           uri = NULL;
