@@ -39,20 +39,20 @@ static void pager_buttons_set_property               (GObject       *object,
                                                       GParamSpec    *pspec);
 static void pager_buttons_finalize                   (GObject       *object);
 static void pager_buttons_queue_rebuild              (PagerButtons  *pager);
-static void pager_buttons_screen_workspace_changed   (WnckScreen    *screen,
-                                                      WnckWorkspace *previous_workspace,
+static void pager_buttons_screen_workspace_changed   (XfwWorkspaceGroup *group,
+                                                      XfwWorkspace  *previous_workspace,
                                                       PagerButtons  *pager);
-static void pager_buttons_screen_workspace_created   (WnckScreen    *screen,
-                                                      WnckWorkspace *created_workspace,
+static void pager_buttons_screen_workspace_created   (XfwWorkspaceGroup *group,
+                                                      XfwWorkspace  *created_workspace,
                                                       PagerButtons  *pager);
-static void pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
-                                                      WnckWorkspace *destroyed_workspace,
+static void pager_buttons_screen_workspace_destroyed (XfwWorkspaceGroup *group,
+                                                      XfwWorkspace  *destroyed_workspace,
                                                       PagerButtons  *pager);
-static void pager_buttons_screen_viewports_changed   (WnckScreen    *screen,
+static void pager_buttons_screen_viewports_changed   (XfwWorkspaceGroup *group,
                                                       PagerButtons  *pager);
 static void pager_buttons_workspace_button_toggled   (GtkWidget     *button,
-                                                      WnckWorkspace *workspace);
-static void pager_buttons_workspace_button_label     (WnckWorkspace *workspace,
+                                                      XfwWorkspace  *workspace);
+static void pager_buttons_workspace_button_label     (XfwWorkspace  *workspace,
                                                       GtkWidget     *label);
 static void pager_buttons_viewport_button_toggled    (GtkWidget     *button,
                                                       PagerButtons  *pager);
@@ -72,8 +72,8 @@ struct _PagerButtons
 
   guint           rebuild_id;
 
-  WnckScreen     *wnck_screen;
-  gint            token;
+  XfwScreen      *xfw_screen;
+  XfwWorkspaceGroup *workspace_group;
 
   gint            rows;
   gboolean        numbering;
@@ -116,7 +116,7 @@ pager_buttons_class_init (PagerButtonsClass *klass)
                                    PROP_SCREEN,
                                    g_param_spec_object ("screen",
                                                          NULL, NULL,
-                                                         WNCK_TYPE_SCREEN,
+                                                         XFW_TYPE_SCREEN,
                                                          G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS
                                                          | G_PARAM_CONSTRUCT_ONLY));
 
@@ -149,8 +149,7 @@ static void
 pager_buttons_init (PagerButtons *pager)
 {
   pager->rows = 1;
-  pager->wnck_screen = NULL;
-  pager->token = 0;
+  pager->xfw_screen = NULL;
   pager->orientation = GTK_ORIENTATION_HORIZONTAL;
   pager->numbering = FALSE;
   pager->buttons = NULL;
@@ -202,20 +201,23 @@ pager_buttons_set_property (GObject      *object,
                             GParamSpec   *pspec)
 {
   PagerButtons *pager = XFCE_PAGER_BUTTONS (object);
+  XfwWorkspaceManager *manager;
 
   switch (prop_id)
     {
     case PROP_SCREEN:
-      pager->wnck_screen = g_value_dup_object (value);
-      panel_return_if_fail (WNCK_IS_SCREEN (pager->wnck_screen));
+      pager->xfw_screen = g_value_dup_object (value);
+      panel_return_if_fail (XFW_IS_SCREEN (pager->xfw_screen));
+      manager = xfw_screen_get_workspace_manager (pager->xfw_screen);
+      pager->workspace_group = xfw_workspace_manager_list_workspace_groups (manager)->data;
 
-      g_signal_connect (G_OBJECT (pager->wnck_screen), "active-workspace-changed",
+      g_signal_connect (G_OBJECT (pager->workspace_group), "active-workspace-changed",
           G_CALLBACK (pager_buttons_screen_workspace_changed), pager);
-      g_signal_connect (G_OBJECT (pager->wnck_screen), "workspace-created",
+      g_signal_connect (G_OBJECT (pager->workspace_group), "workspace-created",
           G_CALLBACK (pager_buttons_screen_workspace_created), pager);
-      g_signal_connect (G_OBJECT (pager->wnck_screen), "workspace-destroyed",
+      g_signal_connect (G_OBJECT (pager->workspace_group), "workspace-destroyed",
           G_CALLBACK (pager_buttons_screen_workspace_destroyed), pager);
-      g_signal_connect (G_OBJECT (pager->wnck_screen), "viewports-changed",
+      g_signal_connect (G_OBJECT (pager->workspace_group), "viewports-changed",
           G_CALLBACK (pager_buttons_screen_viewports_changed), pager);
 
       pager_buttons_queue_rebuild (pager);
@@ -249,19 +251,18 @@ pager_buttons_finalize (GObject *object)
   if (pager->rebuild_id != 0)
     g_source_remove (pager->rebuild_id);
 
-  if (G_LIKELY (pager->wnck_screen != NULL))
+  if (G_LIKELY (pager->xfw_screen != NULL))
     {
-      wnck_screen_release_workspace_layout (pager->wnck_screen, pager->token);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
+      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->workspace_group),
           G_CALLBACK (pager_buttons_screen_workspace_changed), pager);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
+      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->workspace_group),
           G_CALLBACK (pager_buttons_screen_workspace_created), pager);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
+      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->workspace_group),
           G_CALLBACK (pager_buttons_screen_workspace_destroyed), pager);
-      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->wnck_screen),
+      g_signal_handlers_disconnect_by_func (G_OBJECT (pager->workspace_group),
           G_CALLBACK (pager_buttons_screen_viewports_changed), pager);
 
-      g_object_unref (G_OBJECT (pager->wnck_screen));
+      g_object_unref (G_OBJECT (pager->xfw_screen));
     }
 
   g_slist_free (pager->buttons);
@@ -297,24 +298,25 @@ pager_buttons_rebuild_idle (gpointer user_data)
 {
   PagerButtons  *pager = XFCE_PAGER_BUTTONS (user_data);
   GList         *li, *workspaces;
-  WnckWorkspace *active_ws;
+  XfwWorkspace  *active_ws;
   gint           n, n_workspaces;
   gint           rows, cols;
   gint           row, col;
   GtkWidget     *button;
-  WnckWorkspace *workspace = NULL;
+  XfwWorkspace  *workspace = NULL;
   GtkWidget     *panel_plugin;
   GtkWidget     *label;
-  gint           workspace_width, workspace_height = 0;
   gint           screen_width = 0, screen_height = 0;
-  gint           viewport_x, viewport_y;
   gboolean       viewport_mode = FALSE;
   gint           n_viewports = 0;
   gint          *vp_info;
   gchar          text[8];
+  GdkRectangle  *rect = NULL;
+  GdkScreen     *screen;
+  guint          scale_factor;
 
   panel_return_val_if_fail (XFCE_IS_PAGER_BUTTONS (pager), FALSE);
-  panel_return_val_if_fail (WNCK_IS_SCREEN (pager->wnck_screen), FALSE);
+  panel_return_val_if_fail (XFW_IS_SCREEN (pager->xfw_screen), FALSE);
 
   gtk_container_foreach (GTK_CONTAINER (pager),
       (GtkCallback) (void (*)(void)) gtk_widget_destroy, NULL);
@@ -322,8 +324,8 @@ pager_buttons_rebuild_idle (gpointer user_data)
   g_slist_free (pager->buttons);
   pager->buttons = NULL;
 
-  active_ws = wnck_screen_get_active_workspace (pager->wnck_screen);
-  workspaces = wnck_screen_get_workspaces (pager->wnck_screen);
+  active_ws = xfw_workspace_group_get_active_workspace (pager->workspace_group);
+  workspaces = xfw_workspace_group_list_workspaces (pager->workspace_group);
   if (workspaces == NULL)
     goto leave;
 
@@ -331,20 +333,20 @@ pager_buttons_rebuild_idle (gpointer user_data)
 
   /* check if the user uses 1 workspace with viewports */
   if (G_UNLIKELY (n_workspaces == 1
-      && wnck_workspace_is_virtual (WNCK_WORKSPACE (workspaces->data))))
+      && xfw_workspace_get_state (workspaces->data) & XFW_WORKSPACE_STATE_VIRTUAL))
     {
-      workspace = WNCK_WORKSPACE (workspaces->data);
-
-      workspace_width = wnck_workspace_get_width (workspace);
-      workspace_height = wnck_workspace_get_height (workspace);
-      screen_width = wnck_screen_get_width (pager->wnck_screen);
-      screen_height = wnck_screen_get_height (pager->wnck_screen);
+      workspace = XFW_WORKSPACE (workspaces->data);
+      rect = xfw_workspace_get_geometry (workspace);
+      scale_factor = gdk_window_get_scale_factor (gtk_widget_get_window (GTK_WIDGET (pager)));
+      screen = gdk_screen_get_default ();
+      screen_width = panel_screen_get_width (screen) * scale_factor;
+      screen_height = panel_screen_get_height (screen) * scale_factor;
 
       /* we only support viewports that are equally spread */
-      if ((workspace_width % screen_width) == 0
-          && (workspace_height % screen_height) == 0)
+      if ((rect->width % screen_width) == 0
+          && (rect->height % screen_height) == 0)
         {
-          n_viewports = (workspace_width / screen_width) * (workspace_height / screen_height);
+          n_viewports = (rect->width / screen_width) * (rect->height / screen_height);
 
           rows = CLAMP (1, pager->rows, n_viewports);
           cols = n_workspaces / rows;
@@ -356,7 +358,7 @@ pager_buttons_rebuild_idle (gpointer user_data)
       else
         {
           g_warning ("only viewports with equally distributed screens are supported: %dx%d & %dx%d",
-                     workspace_width, workspace_height, screen_width, screen_height);
+                     rect->width, rect->height, screen_width, screen_height);
 
           goto workspace_layout;
         }
@@ -373,27 +375,24 @@ pager_buttons_rebuild_idle (gpointer user_data)
 
   /* set workspace layout so changing workspace and moving windows between workspaces
    * via keyboard shortcuts or dnd work correctly in all directions */
-  pager->token = wnck_screen_try_set_workspace_layout (pager->wnck_screen, pager->token, rows, 0);
+  xfw_workspace_group_set_layout (pager->workspace_group, rows, 0, NULL);
 
   panel_plugin = gtk_widget_get_ancestor (GTK_WIDGET (pager), XFCE_TYPE_PANEL_PLUGIN);
 
   if (G_UNLIKELY (viewport_mode))
     {
-      panel_return_val_if_fail (WNCK_IS_WORKSPACE (workspace), FALSE);
-
-      viewport_x = wnck_workspace_get_viewport_x (workspace);
-      viewport_y = wnck_workspace_get_viewport_y (workspace);
+      panel_return_val_if_fail (XFW_IS_WORKSPACE (workspace), FALSE);
 
       for (n = 0; n < n_viewports; n++)
         {
           vp_info = g_new0 (gint, N_INFOS);
-          vp_info[VIEWPORT_X] = (n % (workspace_height / screen_height)) * screen_width;
-          vp_info[VIEWPORT_Y] = (n / (workspace_height / screen_height)) * screen_height;
+          vp_info[VIEWPORT_X] = (n % (rect->height / screen_height)) * screen_width;
+          vp_info[VIEWPORT_Y] = (n / (rect->height / screen_height)) * screen_height;
 
           button = xfce_panel_create_toggle_button ();
           gtk_widget_add_events (GTK_WIDGET (button), GDK_SCROLL_MASK | GDK_SMOOTH_SCROLL_MASK);
-          if (viewport_x >= vp_info[VIEWPORT_X] && viewport_x < vp_info[VIEWPORT_X] + screen_width
-              && viewport_y >= vp_info[VIEWPORT_Y] && viewport_y < vp_info[VIEWPORT_Y] + screen_height)
+          if (rect->x >= vp_info[VIEWPORT_X] && rect->x < vp_info[VIEWPORT_X] + screen_width
+              && rect->y >= vp_info[VIEWPORT_Y] && rect->y < vp_info[VIEWPORT_Y] + screen_height)
             gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
           g_signal_connect (G_OBJECT (button), "toggled",
               G_CALLBACK (pager_buttons_viewport_button_toggled), pager);
@@ -431,7 +430,7 @@ pager_buttons_rebuild_idle (gpointer user_data)
     {
       for (li = workspaces, n = 0; li != NULL; li = li->next, n++)
         {
-          workspace = WNCK_WORKSPACE (li->data);
+          workspace = XFW_WORKSPACE (li->data);
 
           button = xfce_panel_create_toggle_button ();
           gtk_widget_add_events (GTK_WIDGET (button), GDK_SCROLL_MASK | GDK_SMOOTH_SCROLL_MASK);
@@ -504,22 +503,22 @@ pager_buttons_queue_rebuild (PagerButtons *pager)
 
 
 static void
-pager_buttons_screen_workspace_changed (WnckScreen    *screen,
-                                        WnckWorkspace *previous_workspace,
-                                        PagerButtons  *pager)
+pager_buttons_screen_workspace_changed (XfwWorkspaceGroup *group,
+                                        XfwWorkspace *previous_workspace,
+                                        PagerButtons *pager)
 {
   gint           active = -1, n;
-  WnckWorkspace *active_ws;
+  XfwWorkspace  *active_ws;
   GSList        *li;
 
-  panel_return_if_fail (WNCK_IS_SCREEN (screen));
-  panel_return_if_fail (previous_workspace == NULL || WNCK_IS_WORKSPACE (previous_workspace));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (group));
+  panel_return_if_fail (previous_workspace == NULL || XFW_IS_WORKSPACE (previous_workspace));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
-  panel_return_if_fail (pager->wnck_screen == screen);
+  panel_return_if_fail (pager->workspace_group == group);
 
-  active_ws = wnck_screen_get_active_workspace (screen);
+  active_ws = xfw_workspace_group_get_active_workspace (group);
   if (G_LIKELY (active_ws != NULL))
-    active = wnck_workspace_get_number (active_ws);
+    active = xfw_workspace_get_number (active_ws);
 
   for (li = pager->buttons, n = 0; li != NULL; li = li->next, n++)
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (li->data), n == active);
@@ -528,14 +527,14 @@ pager_buttons_screen_workspace_changed (WnckScreen    *screen,
 
 
 static void
-pager_buttons_screen_workspace_created (WnckScreen    *screen,
-                                        WnckWorkspace *created_workspace,
-                                        PagerButtons  *pager)
+pager_buttons_screen_workspace_created (XfwWorkspaceGroup *group,
+                                        XfwWorkspace *created_workspace,
+                                        PagerButtons *pager)
 {
-  panel_return_if_fail (WNCK_IS_SCREEN (screen));
-  panel_return_if_fail (WNCK_IS_WORKSPACE (created_workspace));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (group));
+  panel_return_if_fail (XFW_IS_WORKSPACE (created_workspace));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
-  panel_return_if_fail (pager->wnck_screen == screen);
+  panel_return_if_fail (pager->workspace_group == group);
 
   pager_buttons_queue_rebuild (pager);
 }
@@ -543,14 +542,14 @@ pager_buttons_screen_workspace_created (WnckScreen    *screen,
 
 
 static void
-pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
-                                          WnckWorkspace *destroyed_workspace,
-                                          PagerButtons  *pager)
+pager_buttons_screen_workspace_destroyed (XfwWorkspaceGroup *group,
+                                          XfwWorkspace *destroyed_workspace,
+                                          PagerButtons *pager)
 {
-  panel_return_if_fail (WNCK_IS_SCREEN (screen));
-  panel_return_if_fail (WNCK_IS_WORKSPACE (destroyed_workspace));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (group));
+  panel_return_if_fail (XFW_IS_WORKSPACE (destroyed_workspace));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
-  panel_return_if_fail (pager->wnck_screen == screen);
+  panel_return_if_fail (pager->workspace_group == group);
 
   pager_buttons_queue_rebuild (pager);
 }
@@ -558,12 +557,12 @@ pager_buttons_screen_workspace_destroyed (WnckScreen    *screen,
 
 
 static void
-pager_buttons_screen_viewports_changed (WnckScreen    *screen,
-                                        PagerButtons  *pager)
+pager_buttons_screen_viewports_changed (XfwWorkspaceGroup *group,
+                                        PagerButtons *pager)
 {
-  panel_return_if_fail (WNCK_IS_SCREEN (screen));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (group));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
-  panel_return_if_fail (pager->wnck_screen == screen);
+  panel_return_if_fail (pager->workspace_group == group);
 
   /* yes we are extremely lazy here, but this event is
    * also emitted when the viewport setup changes... */
@@ -574,31 +573,31 @@ pager_buttons_screen_viewports_changed (WnckScreen    *screen,
 
 
 static void
-pager_buttons_workspace_button_label (WnckWorkspace *workspace,
-                                      GtkWidget     *label)
+pager_buttons_workspace_button_label (XfwWorkspace *workspace,
+                                      GtkWidget *label)
 {
   const gchar *name;
   gchar       *utf8 = NULL, *name_fallback = NULL, *name_num = NULL;
   gboolean     numbering;
 
-  panel_return_if_fail (WNCK_IS_WORKSPACE (workspace));
+  panel_return_if_fail (XFW_IS_WORKSPACE (workspace));
   panel_return_if_fail (GTK_IS_LABEL (label));
 
   numbering = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (label), "numbering"));
 
   /* try to get an utf-8 valid name */
-  name = wnck_workspace_get_name (workspace);
-  if (!panel_str_is_empty (name)
+  name = xfw_workspace_get_name (workspace);
+  if (!xfce_str_is_empty (name)
       && !g_utf8_validate (name, -1, NULL))
     name = utf8 = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
 
-  if (panel_str_is_empty (name))
+  if (xfce_str_is_empty (name))
     name = name_fallback = g_strdup_printf (_("Workspace %d"),
-                                            wnck_workspace_get_number (workspace) + 1);
+                                            xfw_workspace_get_number (workspace) + 1);
 
   if (numbering)
     name = name_num = g_strdup_printf ("%d - %s",
-                                       wnck_workspace_get_number (workspace) + 1,
+                                       xfw_workspace_get_number (workspace) + 1,
                                        name);
 
   gtk_label_set_text (GTK_LABEL (label), name);
@@ -612,18 +611,20 @@ pager_buttons_workspace_button_label (WnckWorkspace *workspace,
 
 static void
 pager_buttons_workspace_button_toggled (GtkWidget     *button,
-                                        WnckWorkspace *workspace)
+                                        XfwWorkspace  *workspace)
 {
-  WnckWorkspace *active_ws;
+  PagerButtons *pager;
+  XfwWorkspace *active_ws;
 
   panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (button));
-  panel_return_if_fail (WNCK_IS_WORKSPACE (workspace));
+  panel_return_if_fail (XFW_IS_WORKSPACE (workspace));
 
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
     {
-      active_ws = wnck_screen_get_active_workspace (wnck_workspace_get_screen (workspace));
+      pager = XFCE_PAGER_BUTTONS (gtk_widget_get_ancestor (button, XFCE_TYPE_PAGER_BUTTONS));
+      active_ws = xfw_workspace_group_get_active_workspace (pager->workspace_group);
       if (active_ws != workspace)
-        wnck_workspace_activate (workspace, gtk_get_current_event_time ());
+        xfw_workspace_activate (workspace, NULL);
     }
 }
 
@@ -637,23 +638,22 @@ pager_buttons_viewport_button_toggled (GtkWidget    *button,
 
   panel_return_if_fail (GTK_IS_TOGGLE_BUTTON (button));
   panel_return_if_fail (XFCE_IS_PAGER_BUTTONS (pager));
-  panel_return_if_fail (WNCK_IS_SCREEN (pager->wnck_screen));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (pager->workspace_group));
 
   vp_info = g_object_get_data (G_OBJECT (button), "viewport-info");
   if (G_UNLIKELY (vp_info == NULL))
     return;
 
-  wnck_screen_move_viewport (pager->wnck_screen,
-                             vp_info[VIEWPORT_X],
-                             vp_info[VIEWPORT_Y]);
+  xfw_workspace_group_move_viewport (pager->workspace_group, vp_info[VIEWPORT_X],
+                                     vp_info[VIEWPORT_Y], NULL);
 }
 
 
 
 GtkWidget *
-pager_buttons_new (WnckScreen *screen)
+pager_buttons_new (XfwScreen *screen)
 {
-  panel_return_val_if_fail (WNCK_IS_SCREEN (screen), NULL);
+  panel_return_val_if_fail (XFW_IS_SCREEN (screen), NULL);
 
   return g_object_new (XFCE_TYPE_PAGER_BUTTONS,
                        "screen", screen, NULL);
