@@ -26,7 +26,14 @@
 #endif
 
 #include <gtk/gtk.h>
+#include <libxfce4windowing/libxfce4windowing.h>
+#ifdef HAVE_LIBWNCK
 #include <libwnck/libwnck.h>
+#define pager_plugin_miniature_view_is_supported() GDK_IS_X11_DISPLAY (gdk_display_get_default ())
+#else
+#define pager_plugin_miniature_view_is_supported() FALSE
+#endif
+
 #include <libxfce4panel/libxfce4panel.h>
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4ui/libxfce4ui.h>
@@ -55,12 +62,14 @@ static void     pager_plugin_set_property                 (GObject           *ob
                                                            GParamSpec        *pspec);
 static gboolean pager_plugin_scroll_event                 (GtkWidget         *widget,
                                                            GdkEventScroll    *event);
+#ifdef HAVE_LIBWNCK
 static void     pager_plugin_drag_begin_event             (GtkWidget         *widget,
                                                            GdkDragContext    *context,
                                                            gpointer           user_data);
 static void     pager_plugin_drag_end_event               (GtkWidget         *widget,
                                                            GdkDragContext    *context,
                                                            gpointer           user_data);
+#endif
 static void     pager_plugin_screen_changed               (GtkWidget         *widget,
                                                            GdkScreen         *previous_screen);
 static void     pager_plugin_construct                    (XfcePanelPlugin   *panel_plugin);
@@ -74,8 +83,7 @@ static void     pager_plugin_mode_changed                 (XfcePanelPlugin     *
 static void     pager_plugin_configure_workspace_settings (GtkWidget         *button);
 static void     pager_plugin_configure_plugin             (XfcePanelPlugin   *panel_plugin);
 static gpointer pager_plugin_get_master_plugin            (PagerPlugin       *plugin);
-static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin,
-                                                           gpointer           screen);
+static void     pager_plugin_screen_layout_changed        (PagerPlugin       *plugin);
 static void     pager_plugin_get_preferred_width          (GtkWidget           *widget,
                                                            gint                *minimum_width,
                                                            gint                *natural_width);
@@ -104,10 +112,8 @@ struct _PagerPlugin
 
   GtkWidget     *pager;
 
-  WnckScreen    *wnck_screen;
-#if WNCK_CHECK_VERSION (43, 0, 0)
-  WnckHandle    *wnck_handle;
-#endif
+  XfwScreen     *xfw_screen;
+  XfwWorkspaceGroup *workspace_group;
 
   /* settings */
   guint          scrolling : 1;
@@ -210,18 +216,15 @@ pager_plugin_init (PagerPlugin *plugin)
 {
   PagerPlugin *master_plugin;
 
-  plugin->wnck_screen = NULL;
+  plugin->xfw_screen = NULL;
   plugin->scrolling = TRUE;
   plugin->wrap_workspaces = FALSE;
-  plugin->miniature_view = TRUE;
+  plugin->miniature_view = pager_plugin_miniature_view_is_supported ();
   plugin->numbering = FALSE;
   plugin->ratio = 1.0;
   plugin->pager = NULL;
   plugin->sync_idle_id = 0;
   plugin->sync_wait = TRUE;
-#if WNCK_CHECK_VERSION (43, 0, 0)
-  plugin->wnck_handle = wnck_handle_new (WNCK_CLIENT_TYPE_PAGER);
-#endif
 
   master_plugin = pager_plugin_get_master_plugin (plugin);
   if (master_plugin == NULL)
@@ -292,9 +295,10 @@ pager_plugin_set_property (GObject      *object,
       break;
 
     case PROP_MINIATURE_VIEW:
-      plugin->miniature_view = g_value_get_boolean (value);
-      if (plugin->wnck_screen != NULL)
-        pager_plugin_screen_layout_changed (plugin, NULL);
+      plugin->miniature_view = g_value_get_boolean (value)
+                               && pager_plugin_miniature_view_is_supported ();
+      if (plugin->xfw_screen != NULL)
+        pager_plugin_screen_layout_changed (plugin);
       break;
 
     case PROP_ROWS:
@@ -311,15 +315,17 @@ pager_plugin_set_property (GObject      *object,
         {
           /* set n_rows for master plugin and consequently workspace layout:
            * this is delayed in both cases */
+#ifdef HAVE_LIBWNCK
           if (plugin->miniature_view)
             wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
           else
+#endif
             pager_buttons_set_n_rows (XFCE_PAGER_BUTTONS (plugin->pager), plugin->rows);
 
           /* set n_rows for other plugins: this will queue a pager re-creation */
           for (GSList *lp = plugin_list; lp != NULL; lp = lp->next)
             if (lp->data != plugin
-                && XFCE_PAGER_PLUGIN (lp->data)->wnck_screen == plugin->wnck_screen)
+                && XFCE_PAGER_PLUGIN (lp->data)->xfw_screen == plugin->xfw_screen)
               g_object_set (lp->data, "rows", plugin->rows, NULL);
         }
       else
@@ -331,7 +337,7 @@ pager_plugin_set_property (GObject      *object,
               g_object_set (master_plugin, "rows", rows, NULL);
             }
           else
-            pager_plugin_screen_layout_changed (plugin, NULL);
+            pager_plugin_screen_layout_changed (plugin);
         }
       break;
 
@@ -394,16 +400,16 @@ pager_plugin_scroll_event (GtkWidget      *widget,
                            GdkEventScroll *event)
 {
   PagerPlugin        *plugin = XFCE_PAGER_PLUGIN (widget);
-  WnckWorkspace      *active_ws;
-  WnckWorkspace      *new_ws;
+  XfwWorkspace       *active_ws;
+  XfwWorkspace       *new_ws;
   gint                active_n;
   gint                n_workspaces;
   GdkScrollDirection  scrolling_direction;
 
-  panel_return_val_if_fail (WNCK_IS_SCREEN (plugin->wnck_screen), FALSE);
+  panel_return_val_if_fail (XFW_IS_SCREEN (plugin->xfw_screen), FALSE);
 
   /* leave when scrolling is not enabled */
-  if (plugin->scrolling == FALSE)
+  if (! plugin->scrolling)
     return TRUE;
 
   if (event->direction != GDK_SCROLL_SMOOTH)
@@ -422,8 +428,8 @@ pager_plugin_scroll_event (GtkWidget      *widget,
       return TRUE;
     }
 
-  active_ws = wnck_screen_get_active_workspace (plugin->wnck_screen);
-  active_n = wnck_workspace_get_number (active_ws);
+  active_ws = xfw_workspace_group_get_active_workspace (plugin->workspace_group);
+  active_n = xfw_workspace_get_number (active_ws);
 
   if (scrolling_direction == GDK_SCROLL_UP
       || scrolling_direction == GDK_SCROLL_LEFT)
@@ -431,9 +437,9 @@ pager_plugin_scroll_event (GtkWidget      *widget,
   else
     active_n++;
 
-  n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen) - 1;
+  n_workspaces = xfw_workspace_group_get_workspace_count (plugin->workspace_group) - 1;
 
-  if (plugin->wrap_workspaces == TRUE)
+  if (plugin->wrap_workspaces)
   {
     /* wrap around */
     if (active_n < 0)
@@ -447,15 +453,16 @@ pager_plugin_scroll_event (GtkWidget      *widget,
     return TRUE;
   }
 
-  new_ws = wnck_screen_get_workspace (plugin->wnck_screen, active_n);
+  new_ws = g_list_nth_data (xfw_workspace_group_list_workspaces (plugin->workspace_group), active_n);
   if (new_ws != NULL && active_ws != new_ws)
-    wnck_workspace_activate (new_ws, event->time);
+    xfw_workspace_activate (new_ws, NULL);
 
   return TRUE;
 }
 
 
 
+#ifdef HAVE_LIBWNCK
 static void
 pager_plugin_drag_begin_event (GtkWidget      *widget,
                                GdkDragContext *context,
@@ -482,40 +489,46 @@ pager_plugin_drag_end_event (GtkWidget      *widget,
 
 
 
-static gpointer
-pager_plugin_get_master_plugin (PagerPlugin *plugin)
-{
-  /* one master plugin per WnckScreen */
-  for (GSList *lp = plugin_list; lp != NULL; lp = lp->next)
-    if (XFCE_PAGER_PLUGIN (lp->data)->wnck_screen == plugin->wnck_screen)
-      return lp->data;
-
-  return NULL;
-}
-
-
-
 static void
 pager_plugin_set_ratio (PagerPlugin *plugin)
 {
-  WnckWorkspace *workspace;
+  XfwWorkspace *workspace;
+  GdkScreen *screen = gdk_screen_get_default ();
 
-  g_signal_handlers_disconnect_by_func (plugin->wnck_screen, pager_plugin_set_ratio, plugin);
+  g_signal_handlers_disconnect_by_func (plugin->xfw_screen, pager_plugin_set_ratio, plugin);
 
-  workspace = wnck_screen_get_active_workspace (plugin->wnck_screen);
+  workspace = xfw_workspace_group_get_active_workspace (plugin->workspace_group);
   if (workspace == NULL)
     {
       /* this is the right signal to get relevant information about the virtual
        * nature of active workspace, instead of "active-workspace-changed" */
-      g_signal_connect_swapped (plugin->wnck_screen, "window-manager-changed",
+      g_signal_connect_swapped (plugin->xfw_screen, "window-manager-changed",
                                 G_CALLBACK (pager_plugin_set_ratio), plugin);
       return;
     }
 
-  plugin->ratio = (gfloat) wnck_screen_get_width (plugin->wnck_screen)
-                  / (gfloat) wnck_screen_get_height (plugin->wnck_screen);
-  if (wnck_workspace_is_virtual (workspace))
-    plugin->ratio *= wnck_workspace_get_width (workspace) / wnck_screen_get_width (plugin->wnck_screen);
+  if (xfw_workspace_get_state (workspace) & XFW_WORKSPACE_STATE_VIRTUAL)
+    {
+      GdkRectangle *rect = xfw_workspace_get_geometry (workspace);
+      plugin->ratio = (gfloat) rect->width / (gfloat) rect->height;
+    }
+  else
+    plugin->ratio = (gfloat) panel_screen_get_width (screen)
+                    / (gfloat) panel_screen_get_height (screen);
+}
+#endif
+
+
+
+static gpointer
+pager_plugin_get_master_plugin (PagerPlugin *plugin)
+{
+  /* one master plugin per XfwScreen  */
+  for (GSList *lp = plugin_list; lp != NULL; lp = lp->next)
+    if (XFCE_PAGER_PLUGIN (lp->data)->xfw_screen == plugin->xfw_screen)
+      return lp->data;
+
+  return NULL;
 }
 
 
@@ -534,7 +547,7 @@ pager_plugin_screen_layout_changed_idle (gpointer data)
       return TRUE;
     }
 
-  pager_plugin_screen_layout_changed (plugin, NULL);
+  pager_plugin_screen_layout_changed (plugin);
   plugin->sync_wait = TRUE;
   plugin->sync_idle_id = 0;
 
@@ -544,19 +557,17 @@ pager_plugin_screen_layout_changed_idle (gpointer data)
 
 
 static void
-pager_plugin_screen_layout_changed (PagerPlugin *plugin,
-                                    gpointer screen)
+pager_plugin_screen_layout_changed (PagerPlugin *plugin)
 {
   XfcePanelPluginMode mode;
   GtkOrientation      orientation;
 
   panel_return_if_fail (XFCE_IS_PAGER_PLUGIN (plugin));
-  panel_return_if_fail (WNCK_IS_SCREEN (plugin->wnck_screen));
+  panel_return_if_fail (XFW_IS_SCREEN (plugin->xfw_screen));
 
   /* changing workspace layout is delayed in Libwnck code, so we have to give time
    * to the master plugin request to be processed */
-  if ((plugin != pager_plugin_get_master_plugin (plugin) || screen != NULL)
-      && plugin->sync_idle_id == 0)
+  if (plugin != pager_plugin_get_master_plugin (plugin) && plugin->sync_idle_id == 0)
     {
       plugin->sync_idle_id =
         g_idle_add_full (G_PRIORITY_LOW, pager_plugin_screen_layout_changed_idle, plugin, NULL);
@@ -564,27 +575,21 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin,
     }
 
   if (G_UNLIKELY (plugin->pager != NULL))
-    {
-      gtk_widget_destroy (GTK_WIDGET (plugin->pager));
-      wnck_screen_force_update (plugin->wnck_screen);
-    }
+    gtk_widget_destroy (GTK_WIDGET (plugin->pager));
 
   mode = xfce_panel_plugin_get_mode (XFCE_PANEL_PLUGIN (plugin));
   orientation =
     (mode != XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
     GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
 
+#ifdef HAVE_LIBWNCK
   if (plugin->miniature_view)
     {
       pager_plugin_set_ratio (plugin);
 
-#if WNCK_CHECK_VERSION (43, 0, 0)
-      /* using a handle allows in particular the pager not to be affected by a possible
-       * change of default wnck icon size in other plugins */
-      plugin->pager = wnck_pager_new_with_handle (plugin->wnck_handle);
-#else
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
       plugin->pager = wnck_pager_new ();
-#endif
+G_GNUC_END_IGNORE_DEPRECATIONS
       g_signal_connect_after (G_OBJECT (plugin->pager), "drag-begin",
                               G_CALLBACK (pager_plugin_drag_begin_event), plugin);
       g_signal_connect_after (G_OBJECT (plugin->pager), "drag-end",
@@ -601,8 +606,9 @@ pager_plugin_screen_layout_changed (PagerPlugin *plugin,
       wnck_pager_set_n_rows (WNCK_PAGER (plugin->pager), plugin->rows);
     }
   else
+#endif
     {
-      plugin->pager = pager_buttons_new (plugin->wnck_screen);
+      plugin->pager = pager_buttons_new (plugin->xfw_screen);
       pager_buttons_set_n_rows (XFCE_PAGER_BUTTONS (plugin->pager), plugin->rows);
       pager_buttons_set_orientation (XFCE_PAGER_BUTTONS (plugin->pager), orientation);
       pager_buttons_set_numbering (XFCE_PAGER_BUTTONS (plugin->pager), plugin->numbering);
@@ -625,27 +631,31 @@ pager_plugin_screen_changed (GtkWidget *widget,
                              GdkScreen *previous_screen)
 {
   PagerPlugin *plugin = XFCE_PAGER_PLUGIN (widget);
-  GdkScreen   *screen;
-  WnckScreen  *wnck_screen;
+  GdkScreen *screen;
+  XfwScreen *xfw_screen;
+  XfwWorkspaceManager *manager;
 
-  screen = gtk_widget_get_screen (widget);
-  wnck_screen = panel_wnck_screen_get (panel_screen_get_number (screen));
+  xfw_screen = xfw_screen_get_default ();
 
-  if (plugin->wnck_screen != wnck_screen)
+  if (plugin->xfw_screen != xfw_screen)
     {
-      plugin->wnck_screen = wnck_screen;
+      if (plugin->xfw_screen != NULL)
+        g_object_unref (plugin->xfw_screen);
 
-      pager_plugin_screen_layout_changed (plugin, NULL);
+      plugin->xfw_screen = xfw_screen;
+      manager = xfw_screen_get_workspace_manager (xfw_screen);
+      plugin->workspace_group = xfw_workspace_manager_list_workspace_groups (manager)->data;
 
+      pager_plugin_screen_layout_changed (plugin);
+
+      screen = gdk_screen_get_default ();
       g_signal_connect_object (G_OBJECT (screen), "monitors-changed",
          G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
       g_signal_connect_object (G_OBJECT (screen), "size-changed",
          G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
-      g_signal_connect_object (G_OBJECT (wnck_screen), "window-manager-changed",
-         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
-      g_signal_connect_object (G_OBJECT (wnck_screen), "viewports-changed",
-         G_CALLBACK (pager_plugin_screen_layout_changed), plugin, G_CONNECT_SWAPPED);
     }
+  else
+    g_object_unref (xfw_screen);
 }
 
 
@@ -696,13 +706,11 @@ pager_plugin_free_data (XfcePanelPlugin *panel_plugin)
   g_signal_handlers_disconnect_by_func (G_OBJECT (plugin),
       pager_plugin_screen_changed, NULL);
 
-#if WNCK_CHECK_VERSION (43, 0, 0)
-  g_object_unref (plugin->wnck_handle);
-#endif
-
   plugin_list = g_slist_remove (plugin_list, plugin);
   if (plugin->sync_idle_id != 0)
     g_source_remove (plugin->sync_idle_id);
+
+  g_clear_object (&plugin->xfw_screen);
 }
 
 
@@ -733,9 +741,11 @@ pager_plugin_mode_changed (XfcePanelPlugin     *panel_plugin,
     (mode != XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
     GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
 
+#ifdef HAVE_LIBWNCK
   if (plugin->miniature_view)
     wnck_pager_set_orientation (WNCK_PAGER (plugin->pager), orientation);
   else
+#endif
     pager_buttons_set_orientation (XFCE_PAGER_BUTTONS (plugin->pager), orientation);
 }
 
@@ -769,20 +779,42 @@ pager_plugin_configure_workspace_settings (GtkWidget *button)
 
 
 static void
-pager_plugin_configure_n_workspaces_changed (WnckScreen    *wnck_screen,
-                                             WnckWorkspace *workspace,
-                                             GtkBuilder    *builder)
+pager_plugin_configure_n_workspaces_changed (XfwWorkspaceGroup *group,
+                                             XfwWorkspace *workspace,
+                                             GtkBuilder *builder)
 {
-  GObject *object;
-  gdouble upper, value;
+  GObject       *object;
+  gdouble        upper, value;
+  XfwWorkspace  *active_ws;
+  guint          scale_factor;
+  GtkWidget     *dialog;
+  GdkRectangle  *rect;
+  GdkScreen     *screen = gdk_screen_get_default ();
 
-  panel_return_if_fail (WNCK_IS_SCREEN (wnck_screen));
+  panel_return_if_fail (XFW_IS_WORKSPACE_GROUP (group));
   panel_return_if_fail (GTK_IS_BUILDER (builder));
 
   object = gtk_builder_get_object (builder, "rows");
-  upper = wnck_screen_get_workspace_count (wnck_screen);
+
+  upper = xfw_workspace_group_get_workspace_count (group);
+  if (upper == 1)
+    {
+      /* check if we ware in viewport mode */
+      active_ws = xfw_workspace_group_get_active_workspace (group);
+      if (xfw_workspace_get_state (active_ws) & XFW_WORKSPACE_STATE_VIRTUAL)
+        {
+          /* number of rows * number of columns */
+          dialog = GTK_WIDGET (gtk_builder_get_object (builder, "dialog"));
+          scale_factor = gdk_window_get_scale_factor (gtk_widget_get_window (dialog));
+          rect = xfw_workspace_get_geometry (active_ws);
+          upper = rect->width / (panel_screen_get_width (screen) * scale_factor)
+                  * rect->height / (panel_screen_get_height (screen) * scale_factor);
+        }
+    }
+
   value = MIN (gtk_adjustment_get_value (GTK_ADJUSTMENT (object)), upper);
-  g_object_set (object, "upper", upper, "value", value, NULL);
+
+  g_object_set (G_OBJECT (object), "upper", upper, "value", value, NULL);
 }
 
 
@@ -793,7 +825,7 @@ pager_plugin_configure_destroyed (gpointer  data,
 {
   PagerPlugin *plugin = XFCE_PAGER_PLUGIN (data);
 
-  g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->wnck_screen),
+  g_signal_handlers_disconnect_by_func (G_OBJECT (plugin->workspace_group),
                                         pager_plugin_configure_n_workspaces_changed,
                                         where_the_object_was);
 }
@@ -817,9 +849,9 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
     return;
 
   /* signals to monitor number of workspace changes */
-  g_signal_connect (G_OBJECT (plugin->wnck_screen), "workspace-created",
+  g_signal_connect (G_OBJECT (plugin->workspace_group), "workspace-created",
       G_CALLBACK (pager_plugin_configure_n_workspaces_changed), builder);
-  g_signal_connect (G_OBJECT (plugin->wnck_screen), "workspace-destroyed",
+  g_signal_connect (G_OBJECT (plugin->workspace_group), "workspace-destroyed",
       G_CALLBACK (pager_plugin_configure_n_workspaces_changed), builder);
   g_object_weak_ref (G_OBJECT (builder), pager_plugin_configure_destroyed, plugin);
 
@@ -833,6 +865,7 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
   g_object_bind_property (G_OBJECT (plugin), "miniature-view",
                           G_OBJECT (object), "active",
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
+  gtk_widget_set_sensitive (GTK_WIDGET (object), pager_plugin_miniature_view_is_supported ());
 
   object = gtk_builder_get_object (builder, "rows");
   panel_return_if_fail (GTK_IS_ADJUSTMENT (object));
@@ -866,7 +899,7 @@ pager_plugin_configure_plugin (XfcePanelPlugin *panel_plugin)
                           G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
 
   /* update the rows limit */
-  pager_plugin_configure_n_workspaces_changed (plugin->wnck_screen, NULL, builder);
+  pager_plugin_configure_n_workspaces_changed (plugin->workspace_group, NULL, builder);
 
   gtk_widget_show (GTK_WIDGET (dialog));
 }
@@ -892,7 +925,7 @@ pager_plugin_get_preferred_width (GtkWidget *widget,
     min_width = nat_width = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
   else if (plugin->miniature_view)
     {
-      n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen);
+      n_workspaces = xfw_workspace_group_get_workspace_count (plugin->workspace_group);
       n_cols = MAX (1, (n_workspaces + plugin->rows - 1) / plugin->rows);
       min_width = nat_width = (gint) (xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)) / plugin->rows * plugin->ratio * n_cols);
     }
@@ -923,7 +956,7 @@ pager_plugin_get_preferred_height (GtkWidget *widget,
     min_height = nat_height = xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin));
   else if (plugin->miniature_view)
     {
-      n_workspaces = wnck_screen_get_workspace_count (plugin->wnck_screen);
+      n_workspaces = xfw_workspace_group_get_workspace_count (plugin->workspace_group);
       n_cols = MAX (1, (n_workspaces + plugin->rows - 1) / plugin->rows);
       if (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
         min_height = nat_height = (gint) (xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (plugin)) / plugin->rows / plugin->ratio * n_cols);
