@@ -30,6 +30,7 @@
 #include <libxfce4util/libxfce4util.h>
 #include <libxfce4panel/libxfce4panel.h>
 
+#include <common/panel-private.h>
 #include "sn-dialog.h"
 #include "sn-dialog-ui.h"
 
@@ -70,7 +71,7 @@ G_DEFINE_TYPE (SnDialog, sn_dialog, G_TYPE_OBJECT)
 
 enum
 {
-  COLUMN_ICON,
+  COLUMN_PIXBUF,
   COLUMN_TITLE,
   COLUMN_HIDDEN,
   COLUMN_TIP
@@ -140,7 +141,7 @@ sn_dialog_init (SnDialog *dialog)
 
 static void
 sn_dialog_add_item (SnDialog   *dialog,
-                    GIcon       *icon,
+                    GdkPixbuf   *pixbuf,
                     const gchar *name,
                     const gchar *title,
                     gboolean     hidden)
@@ -154,7 +155,7 @@ sn_dialog_add_item (SnDialog   *dialog,
   /* insert in the store */
   gtk_list_store_append (GTK_LIST_STORE (dialog->store), &iter);
   gtk_list_store_set (GTK_LIST_STORE (dialog->store), &iter,
-                      COLUMN_ICON,    icon,
+                      COLUMN_PIXBUF,  pixbuf,
                       COLUMN_TITLE,   title,
                       COLUMN_HIDDEN,  hidden,
                       COLUMN_TIP,     name,
@@ -165,7 +166,7 @@ sn_dialog_add_item (SnDialog   *dialog,
 
 static void
 sn_dialog_add_legacy_item(SnDialog *dialog,
-                   GIcon *icon,
+                   GdkPixbuf *pixbuf,
                    const gchar *name,
                    const gchar *title,
                    gboolean hidden)
@@ -179,7 +180,7 @@ sn_dialog_add_legacy_item(SnDialog *dialog,
   /* insert in the store */
   gtk_list_store_append(GTK_LIST_STORE(dialog->legacy_store), &iter);
   gtk_list_store_set(GTK_LIST_STORE(dialog->legacy_store), &iter,
-                     COLUMN_ICON, icon,
+                     COLUMN_PIXBUF, pixbuf,
                      COLUMN_TITLE, title,
                      COLUMN_HIDDEN, hidden,
                      COLUMN_TIP, name,
@@ -195,7 +196,7 @@ sn_dialog_update_names (SnDialog *dialog)
   const gchar *name;
   const gchar *title;
   const gchar *icon_name;
-  GIcon       *icon;
+  GdkPixbuf   *pixbuf;
   guint        i;
 
   g_return_if_fail (XFCE_IS_SN_DIALOG (dialog));
@@ -207,7 +208,6 @@ sn_dialog_update_names (SnDialog *dialog)
       name = li->data;
       title = name;
       icon_name = name;
-      icon = NULL;
 
       /* check if we have a better name for the application */
       for (i = 0; i < G_N_ELEMENTS (known_applications); i++)
@@ -220,15 +220,14 @@ sn_dialog_update_names (SnDialog *dialog)
             }
         }
 
-      if (gtk_icon_theme_has_icon (gtk_icon_theme_get_default (), icon_name))
-        icon = g_themed_icon_new (icon_name);
+      pixbuf = xfce_panel_pixbuf_from_source (icon_name, NULL, 22);
 
       /* insert item in the store */
-      sn_dialog_add_item (dialog, icon, name, title,
+      sn_dialog_add_item (dialog, pixbuf, name, title,
                           sn_config_is_hidden (dialog->config, name));
 
-      if (icon != NULL)
-        g_object_unref (G_OBJECT (icon));
+      if (pixbuf != NULL)
+        g_object_unref (G_OBJECT (pixbuf));
     }
 }
 
@@ -241,7 +240,7 @@ sn_dialog_update_legacy_names(SnDialog *dialog)
   const gchar *name;
   const gchar *title;
   const gchar *icon_name;
-  GIcon *icon;
+  GdkPixbuf *pixbuf;
   guint i;
 
   g_return_if_fail(XFCE_IS_SN_DIALOG(dialog));
@@ -253,7 +252,6 @@ sn_dialog_update_legacy_names(SnDialog *dialog)
     name = li->data;
     title = name;
     icon_name = name;
-    icon = NULL;
 
     /* check if we have a better name for the application */
     for (i = 0; i < G_N_ELEMENTS(known_legacy_applications); i++)
@@ -266,15 +264,14 @@ sn_dialog_update_legacy_names(SnDialog *dialog)
       }
     }
 
-    if (gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), icon_name))
-      icon = g_themed_icon_new(icon_name);
+    pixbuf = xfce_panel_pixbuf_from_source(icon_name, NULL, 22);
 
     /* insert item in the store */
-    sn_dialog_add_legacy_item(dialog, icon, name, title,
+    sn_dialog_add_legacy_item(dialog, pixbuf, name, title,
                        sn_config_is_legacy_hidden(dialog->config, name));
 
-    if (icon != NULL)
-      g_object_unref(G_OBJECT(icon));
+    if (pixbuf != NULL)
+      g_object_unref(G_OBJECT(pixbuf));
   }
 }
 
@@ -427,7 +424,7 @@ sn_dialog_swap_rows (SnDialog   *dialog,
                      GtkTreeIter *iter_prev,
                      GtkTreeIter *iter)
 {
-  GIcon     *icon1, *icon2;
+  GdkPixbuf *pixbuf1, *pixbuf2;
   gchar     *title1, *title2;
   gboolean   hidden1, hidden2;
   gchar     *tip1, *tip2;
@@ -437,22 +434,22 @@ sn_dialog_swap_rows (SnDialog   *dialog,
   g_return_if_fail (GTK_IS_LIST_STORE (dialog->store));
 
   gtk_tree_model_get (GTK_TREE_MODEL (dialog->store), iter_prev,
-                      COLUMN_ICON,    &icon1,
+                      COLUMN_PIXBUF,  &pixbuf1,
                       COLUMN_TITLE,   &title1,
                       COLUMN_HIDDEN,  &hidden1,
                       COLUMN_TIP,     &tip1, -1);
   gtk_tree_model_get (GTK_TREE_MODEL (dialog->store), iter,
-                      COLUMN_ICON,    &icon2,
+                      COLUMN_PIXBUF,  &pixbuf2,
                       COLUMN_TITLE,   &title2,
                       COLUMN_HIDDEN,  &hidden2,
                       COLUMN_TIP,     &tip2, -1);
   gtk_list_store_set (GTK_LIST_STORE (dialog->store), iter_prev,
-                      COLUMN_ICON,    icon2,
+                      COLUMN_PIXBUF,  pixbuf2,
                       COLUMN_TITLE,   title2,
                       COLUMN_HIDDEN,  hidden2,
                       COLUMN_TIP,     tip2, -1);
   gtk_list_store_set (GTK_LIST_STORE (dialog->store), iter,
-                      COLUMN_ICON,    icon1,
+                      COLUMN_PIXBUF,  pixbuf1,
                       COLUMN_TITLE,   title1,
                       COLUMN_HIDDEN,  hidden1,
                       COLUMN_TIP,     tip1, -1);
@@ -468,7 +465,7 @@ sn_dialog_legacy_swap_rows(SnDialog *dialog,
                            GtkTreeIter *iter_prev,
                            GtkTreeIter *iter)
 {
-  GIcon *icon1, *icon2;
+  GdkPixbuf *pixbuf1, *pixbuf2;
   gchar *title1, *title2;
   gboolean hidden1, hidden2;
   gchar *tip1, *tip2;
@@ -478,22 +475,22 @@ sn_dialog_legacy_swap_rows(SnDialog *dialog,
   g_return_if_fail(GTK_IS_LIST_STORE(dialog->legacy_store));
 
   gtk_tree_model_get(GTK_TREE_MODEL(dialog->legacy_store), iter_prev,
-                     COLUMN_ICON, &icon1,
+                     COLUMN_PIXBUF, &pixbuf1,
                      COLUMN_TITLE, &title1,
                      COLUMN_HIDDEN, &hidden1,
                      COLUMN_TIP, &tip1, -1);
   gtk_tree_model_get(GTK_TREE_MODEL(dialog->legacy_store), iter,
-                     COLUMN_ICON, &icon2,
+                     COLUMN_PIXBUF, &pixbuf2,
                      COLUMN_TITLE, &title2,
                      COLUMN_HIDDEN, &hidden2,
                      COLUMN_TIP, &tip2, -1);
   gtk_list_store_set(GTK_LIST_STORE(dialog->legacy_store), iter_prev,
-                     COLUMN_ICON, icon2,
+                     COLUMN_PIXBUF, pixbuf2,
                      COLUMN_TITLE, title2,
                      COLUMN_HIDDEN, hidden2,
                      COLUMN_TIP, tip2, -1);
   gtk_list_store_set(GTK_LIST_STORE(dialog->legacy_store), iter,
-                     COLUMN_ICON, icon1,
+                     COLUMN_PIXBUF, pixbuf1,
                      COLUMN_TITLE, title1,
                      COLUMN_HIDDEN, hidden1,
                      COLUMN_TIP, tip1, -1);
@@ -715,6 +712,7 @@ sn_dialog_build (SnDialog *dialog)
     return FALSE;
 
   dialog->builder = gtk_builder_new ();
+  gtk_builder_set_translation_domain (dialog->builder, GETTEXT_PACKAGE);
 
   /* load the builder data into the object */
   if (gtk_builder_add_from_string (dialog->builder, sn_dialog_ui,
@@ -853,6 +851,14 @@ sn_dialog_build (SnDialog *dialog)
       gtk_widget_hide (GTK_WIDGET (object));
 #endif
 
+      if (! GDK_IS_X11_DISPLAY (gdk_display_get_default ()))
+        {
+          object = gtk_builder_get_object (dialog->builder, "systray_frame");
+          gtk_widget_hide (GTK_WIDGET (object));
+          object = gtk_builder_get_object (dialog->builder, "items_stack_switcher");
+          gtk_widget_hide (GTK_WIDGET (object));
+        }
+
       g_object_weak_ref (G_OBJECT (dialog->dialog), sn_dialog_dialog_unref, dialog);
       return TRUE;
     }
