@@ -27,11 +27,7 @@
 #include <string.h>
 #endif
 
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4panel/libxfce4panel.h>
