@@ -22,7 +22,6 @@
 #endif
 
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 
 #include <libxfce4util/libxfce4util.h>
@@ -327,7 +326,7 @@ static gboolean
 systray_plugin_names_get_hidden (SnPlugin      *plugin,
                                  const gchar   *name)
 {
-  if (panel_str_is_empty (name))
+  if (xfce_str_is_empty (name))
     return FALSE;
 
   /* lookup the name in the list */
